#! /usr/bin/ksh
#===============================================================================
#  slserver.sh : 
#       DataDirect SequeLink UNIX installation script
#       Copyright(c) MERANT Inc., 1991-2000 All rights reserved.
#
#===============================================================================

#===============================================================================
#  toolkit.sh : 
#       DataDirect SequeLink UNIX installer
#  	Copyright(c) MERANT Inc., 1997-1999 All rights reserved.
# 
#===============================================================================

#-------------------------------------------------------------------------------
#  Global init
#-------------------------------------------------------------------------------
TEMP_FILE="sequelnk.tmp"
LOGFILE=/dev/null
TARXF="tar -xof"

#===============================================================================
#  Ini functions (rscshell implementation)
#    These functions are case insensitive for <section> and <key>
#    Spaces and tabs are allowed before and/or after '='
#===============================================================================

#-------------------------------------------------------------------------------
#  Ini_Section_Create
#    Creates a section if it does not exists
#  usage : Ini_Section_Create <file> <section> 
#-------------------------------------------------------------------------------
Ini_Section_Create ()
  {
  ./rscshell 1 "$1" "$2"
  }
  
#-------------------------------------------------------------------------------
#  Ini_Section_Delete
#    Deletes a section and al it's keys
#  usage : Ini_Section_Delete <file> <section>
#-------------------------------------------------------------------------------
Ini_Section_Delete ()
  {
  ./rscshell 2 "$1" "$2"
  }

#-------------------------------------------------------------------------------
#  Ini_Section_ListKeys
#  usage : Ini_Section_ListKeys <file> <section>
#-------------------------------------------------------------------------------
Ini_Section_ListKeys()
  {
  ./rscshell 3 "$1" "$2"
  }

#-------------------------------------------------------------------------------
#  Ini_Section_ReadKey
#  usage : Ini_Section_ReadKey <file> <section> <key>
#-------------------------------------------------------------------------------
Ini_Section_ReadKey()
  {
  ./rscshell 4 "$1" "$2" "$3"
  }

#-------------------------------------------------------------------------------
#  Ini_Section_WriteKey
#    Writes a key-value pair in a section. section must exist!
#  usage : Ini_Section_WriteKey <file> <section> <key> <value>
#-------------------------------------------------------------------------------
Ini_Section_WriteKey ()
  {
  ./rscshell 5 "$1" "$2" "$3" "$4"
  }

#-------------------------------------------------------------------------------
#  Ini_Section_DeleteKey
#    Deletes a key-value pair from a section.
#  usage : Ini_Section_DeleteKey <file> <section> <key>
#-------------------------------------------------------------------------------
Ini_Section_DeleteKey ()
  {
  ./rscshell 6 "$1" "$2" "$3"
  }

#===============================================================================
#  Ini functions (Korn shell implementation)
#    These functions are case sensitive for <section> and <key>
#    Spaces and tabs are allowed before and/or after '='
#    Metacharacters for 'sed' are NOT allowed in <section>
#===============================================================================

#-------------------------------------------------------------------------------
#  Ini_Section_ReadMsg
#  usage : Ini_Section_ReadMsg <file> <section>
#-------------------------------------------------------------------------------
Ini_Section_ReadMsg()
{
SECTION="^\[$2\]"
ENDSECTION="^\["
eval sed -ne '/"$SECTION"/,/"$ENDSECTION"/p' "$1" 2>> "$LOGFILE" \
    | grep -v "$ENDSECTION" 
}

#-------------------------------------------------------------------------------
#  Ini_Section_DoCommand
#  usage : Ini_Section_DoCommand <file> <section>
#-------------------------------------------------------------------------------
Ini_Section_DoCommand()
{
SECTION="^\[$2\]"
ENDSECTION="^\["
eval "`eval sed -ne '/$SECTION/,/$ENDSECTION/p' $1 2>> "$LOGFILE" \
    | grep -v $ENDSECTION`"
}

#-------------------------------------------------------------------------------
#  Ini_Section_Create_ksh
#    Creates a section if it does not exists
#  usage : Ini_Section_Create_ksh <file> <section> 
#    special characters in <section> must be escaped!
#-------------------------------------------------------------------------------
Ini_Section_Create_ksh ()
{
SECTION="^\[$2\]"
if [ -z "`grep "$SECTION" "$1" 2> /dev/null`" ] ; then
  echo "[$2]" >> $1
fi
}

#-------------------------------------------------------------------------------
#  Ini_Section_Delete_ksh
#    Deletes a section and al it's keys
#  usage : Ini_Section_Delete_ksh <file> <section>
#    special characters in <section> must be escaped!
#-------------------------------------------------------------------------------
Ini_Section_Delete_ksh ()
{
SECTION="[$2]"
unset INSECTION
rm $TEMP_FILE 2> /dev/null

while read x; do
  if [ "$x" = "$SECTION" ] ; then
    INSECTION="Y"
  elif [ "[" = "${x%%[![]*}" ] ; then
    INSECTION="N"
  fi
  if [ "$INSECTION" != "Y" ] ; then
    echo "$x" >>$TEMP_FILE
  fi
done < $1
cat $TEMP_FILE > $1
rm $TEMP_FILE 2> /dev/null
}

#-------------------------------------------------------------------------------
#  Ini_Section_ListKeys_ksh
#  usage : Ini_Section_ListKeys_ksh <file> <section>
#    special characters in <section> must be escaped twice!
#-------------------------------------------------------------------------------
Ini_Section_ListKeys_ksh()
{
SECTION="^\[$2\]"
ENDSECTION="^\["
eval sed -ne '/"$SECTION"/,/"$ENDSECTION"/p' "$1" 2>> "$LOGFILE" \
    | grep -v "$ENDSECTION" | awk -F= '{print$1}'
}

#-------------------------------------------------------------------------------
#  Ini_Section_ReadKey_ksh
#  usage : Ini_Section_ReadKey_ksh <file> <section> <key>
#    special characters in <section> must be escaped twice!
#-------------------------------------------------------------------------------
Ini_Section_ReadKey_ksh()
{
RET=`eval sed -ne '/"^\[$2\]"/,/"^\["/p' "$1" 2>> "$LOGFILE" \
    | grep -v "^\[" | tr -d "\011" | grep "^$3[ 	]*=" \
    | awk -F= '{print$2}' | tr -d "\011"`
echo ${RET## }
}

#-------------------------------------------------------------------------------
#  Ini_Section_WriteKey_ksh
#    Writes a key-value pair in a section. section must exist!
#  usage : Ini_Section_WriteKey_ksh <file> <section> <key> <value>
#    special characters in <section> must be escaped!
#-------------------------------------------------------------------------------
Ini_Section_WriteKey_ksh ()
{
SECTION="[$2]"
unset INSECTION
rm $TEMP_FILE 2> /dev/null

while read x; 
do
  if [ "$x" = "[$2]" ] ;then
    INSECTION="Y"
    echo "$x" >>$TEMP_FILE
    echo "$3=$4" >>$TEMP_FILE
  elif [ "[" = "${x%%[![]*}" ] ;then
    INSECTION="N"
    echo "$x" >>$TEMP_FILE
  elif [[ "$INSECTION" != "Y" || -z `echo "$x" | tr -d "\011" | grep "^$3[ 	]*="` ]] ;then
    echo "$x" >>$TEMP_FILE
  fi
done < $1

cat $TEMP_FILE > $1
rm $TEMP_FILE 2> /dev/null
}

#-------------------------------------------------------------------------------
#  Ini_Section_DeleteKey_ksh
#    Deletes a key-value pair from a section.
#  usage : Ini_Section_DeleteKey_ksh <file> <section> <key>
#    special characters in <section> must be escaped!
#-------------------------------------------------------------------------------
Ini_Section_DeleteKey_ksh ()
{
SECTION="[$2]"
unset INSECTION
rm $TEMP_FILE 2> /dev/null

while read x; do
  if [ "$x" = "$SECTION" ] ;then
    INSECTION="Y"
  elif [ "[" = "${x%%[![]*}" ] ;then
    INSECTION="N"
  fi
  
  if [[ "$INSECTION" != "Y" || -z `echo "$x" | tr -d "\011" | grep "^$3[ 	]*="` ]] ;then
    echo "$x" >>$TEMP_FILE
  fi
done < $1
cat $TEMP_FILE > $1
rm $TEMP_FILE 2> /dev/null
}

#===============================================================================
#  Miscellaneous functions
#===============================================================================

#-------------------------------------------------------------------------------
#  Log_Create
#-------------------------------------------------------------------------------
Log_Create()
{
let i=1
while :
do
  LOGFILE="$1$i"
  if [ ! -a "$LOGFILE" ] ; then
    break
  fi
  let i=i+1
done
echo                                       >  "$LOGFILE"
Ini_Section_DoCommand "$MSGFILE" LogBanner >> "$LOGFILE"
}

#-------------------------------------------------------------------------------
#  Log_WriteString
#  usage: Log_WriteString <filename> <string>
#-------------------------------------------------------------------------------
Log_WriteString ()
{
eval echo "$2" >> $1
}

#-------------------------------------------------------------------------------
#  Log_WriteVar
#  usage: Log_WriteVar <filename> <varname>
#-------------------------------------------------------------------------------
Log_WriteVar ()
{
eval echo "$2="$"$2" >> $1
}

#-------------------------------------------------------------------------------
#  Dir_Create
#    Creates a directory, if it does not exists
#  usage: Dir_Create <dir> <mode>
#-------------------------------------------------------------------------------
Dir_Create ()
{
if [ ! -d "$1" ]; then
  Log_WriteString "$LOGFILE" "$CREATEDIR $1"
  mkdir -p "$1" 2> $TEMP_FILE
  if [ $? -ne 0 ] ; then
   cat $TEMP_FILE | tee -a "$LOGFILE"
   rm $TEMP_FILE 2> /dev/null
   return 1
  else
   rm $TEMP_FILE 2> /dev/null
   if [ -n "$2" ] ; then
     chmod "$2" "$1" 2> /dev/null
   fi
   return 0
  fi
fi
}


#-------------------------------------------------------------------------------
#  File_Copy
#    Copies a single file
#  usage: File_Copy <source> <target> <overwritemode>
#-------------------------------------------------------------------------------
File_Copy ()
{
error=0
if [ -d "$2" ] ; then
  print -n "`Ini_Section_ReadMsg "$MSGFILE" MsgDirExists`" | tee -a "$LOGFILE"
  echo "$2" | tee -a "$LOGFILE"
  return 1
fi

from=`ls -i "$1" 2> /dev/null | awk '{print $1}'`
to=`ls -i "$2"   2> /dev/null | awk '{print $1}'`
if [ \( -n "$2" \) -a \( "$from" != "$to" \) ] ; then
  Dir_Create "${2%/*}"
  if [ -a "$2" ] ; then
    case "$3" in
    
    # always overwrite
    A|a)
    eval echo "$COPYFILE"          | tee -a "$LOGFILE"
    rm -f "$2";cp -p "$1" "$2"
    error=$?
    break
    ;;
    
    # never overwrite
    N|n)
    Log_WriteString "$LOGFILE" "$SKIPFILE $2"
    return 0
    ;;

    # overwrite older
    O|o)
    if [ ! "$1" -ot "$2" ] ; then
      eval echo "$COPYFILE"        | tee -a "$LOGFILE"
      rm -f "$2";cp -p "$1" "$2"
      error=$?
      break
    else
      Log_WriteString "$LOGFILE" "$SKIPFILE $2"
      return 0
    fi
    ;;
  
    *)
    Ini_Section_ReadMsg "$MSGFILE" MsgOverwrite
    echo "$2"
    QueryYN "`Ini_Section_ReadMsg "$MSGFILE" QueryOverwrite`"
    if [ "$ANSWER" = "$N" ] ; then
      Log_WriteString "$LOGFILE" "$SKIPFILE $2"
      return 0
    else
      eval echo "$COPYFILE"        | tee -a "$LOGFILE"
      rm -f "$2";cp -p "$1" "$2"
      error=$?
      break
    fi
    ;;
    esac
  else
    eval echo "$COPYFILE"          | tee -a "$LOGFILE"
      rm -f "$2";cp -p "$1" "$2"    
    error=$?
  fi
fi

return $error
}

#-------------------------------------------------------------------------------
#  File_Backup
#    Copies a file to a backup copy in the same dir but with extra extension.
#  usage: File_Backup <file> 
#-------------------------------------------------------------------------------
File_Backup ()
{
if [ -a "$1" ] ; then
  let i=1
  while :
  do
    FNAME="$1$i"
    if [ ! -a "$FNAME" ] ; then
      break
    fi
    let i=i+1
  done
  eval echo "$BACKUPFILE $FNAME" | tee -a "$LOGFILE"
  cp "$1" "$FNAME"
  return $?
fi
return 0
}

#-------------------------------------------------------------------------------
#  File_Extract
#    extracts a single file from tar
#  usage: File_Extract <tarfile> <file> <target> <overwritemode>
#  target must be a dir+filename or empty
#  The TARXF variable must be set to the correct command
#-------------------------------------------------------------------------------
File_Extract ()
{
exist=""
extract=""
error=0
if [ -a "$2" ] ; then
  exist=1
  case "$4" in
  
  # always overwrite
  A|a)
  extract=1
  `$TARXF $1 $2`
  error=$?
  break
  ;;

  # never overwrite
  N|n)
  break
  ;;
  
  # overwrite older
  O|o)
  rm -f "$TEMP_FILE";cp -p "$2" "$TEMP_FILE"    2> /dev/null
  `$TARXF $1 $2`
  error=$?
  if [ ! "$2" -ot "$TEMP_FILE" ] ; then
    extract=1
    rm -f "$TEMP_FILE"
  else
    mv -f "$TEMP_FILE" "$2"   2> /dev/null
  fi
  break
  ;;
  
  *)
  Ini_Section_ReadMsg "$MSGFILE" MsgOverwrite
  echo "$2"
  QueryYN "`Ini_Section_ReadMsg "$MSGFILE" QueryOverwrite`"
  if [ "$ANSWER" = "$N" ] ; then
    break
  else
    extract=1
    `$TARXF $1 $2`
    error=$?
    break
  fi
  ;;
  esac
else
  extract=1
  `$TARXF $1 $2`
  error=$?
fi

if [ $error != 0 ] ; then
  return $error
fi

from=`ls -i "$2" 2> /dev/null | awk '{print $1}'`
to=`ls -i "$3"   2> /dev/null | awk '{print $1}'`
if [ -z "$from" ] ; then
  # if the untar didn't worked out, return an error
  Log_WriteString "$LOGFILE" "$NOTFOUND $2"
  return 1
fi

if [ \( -n "$3" \) -a \( "$from" != "$to" \) ] ; then
  # source != destination -> move/copy to target
  Dir_Create "${3%/*}"
  if [ -a "$3" ] ; then
    case "$4" in

    # always overwrite
    A|a)
    break
    ;;
    
    # never overwrite
    N|n)
    Log_WriteString "$LOGFILE" "$SKIPFILE $3"
    if [ -z "$exist" ] ; then
      rm -f "$2"                  2> /dev/null
    fi
    return 0
    ;;

    # overwrite older
    O|o)
    if [ ! "$2" -ot "$3" ] ; then
      break
    else
      Log_WriteString "$LOGFILE" "$SKIPFILE $3"
      if [ -z "$exist" ] ; then
        rm -f "$2"                2> /dev/null
      fi
      return 0
    fi
    ;;

    # interactive (default)  
    *)
    Ini_Section_ReadMsg "$MSGFILE" MsgOverwrite
    echo "$2"
    QueryYN "`Ini_Section_ReadMsg "$MSGFILE" QueryOverwrite`"
    if [ "$ANSWER" = "$N" ] ; then
      Log_WriteString "$LOGFILE" "$SKIPFILE $3"
      if [ -z "$exist" ] ; then
        rm -f "$2"                2> /dev/null
      fi
      return 0
    else
      break
    fi
    ;;
    esac
  fi
    #do the move / copy
    eval echo "$EXTRACTFILE"     | tee -a "$LOGFILE"
    if [ -z "$exist" ] ; then
      mv -f "$2" "$3"
    else
      rm -f "$3";cp -p "$2" "$3"
    fi
    return $?
else  
  # source == destination -> don't move
  if [ -n "$extract" ] ; then
    eval echo "$EXTRACTFILE"     | tee -a "$LOGFILE"
  else
    Log_WriteString "$LOGFILE" "$SKIPFILE $3"
  fi
  return 0
fi

return $error
}

#-------------------------------------------------------------------------------
#  Ini_Section_UntarFiles
#    Untars and moves all files in a section
#  usage : Ini_Section_UntarFiles <inifile> <section> <tarfile> <targetrootdir>
#  !The TARXF variable must be defined
#-------------------------------------------------------------------------------
Ini_Section_UntarFiles()
{
SECTION="^\[$2\]"
ENDSECTION="^\["
for x in `eval sed -ne '/"$SECTION"/,/"$ENDSECTION"/p' $1 2>> "$LOGFILE" \
    | grep -v "$SECTION" | grep -v "$ENDSECTION"`
do
  FROM=`echo $x | awk -F= '{print $1}'`
  if [ -n "$4" ] ; then
    TO="$4/"`echo $x | awk -F= '{print $2}'`
  else
    TO="./`echo $x | awk -F= '{print $2}'`"
  fi
  OVERWR=`echo $x | awk -F= '{print $3}'`
  MODE=`echo $x | awk -F= '{print $4}'`
  while :
  do
    File_Extract "$3" "$FROM" "$TO" "$OVERWR"
    if [ $? -ne 0 ] ; then
      Ini_Section_ReadMsg "$MSGFILE" MsgErrorExtract1 | tee -a "$LOGFILE"
      echo "$FROM"                                    | tee -a "$LOGFILE"
      Ini_Section_ReadMsg "$MSGFILE" MsgErrorExtract2
      QueryYNC_Yes "`Ini_Section_ReadMsg "$MSGFILE" MsgErrorExtract3`"
      case "$ANSWER" in
        $N)
        Log_WriteString "$LOGFILE" "$SKIPFILE $TO"
        break
        ;;
        $Y)
        continue
        ;;
        $C)
        return 1
        ;;
      esac
    fi
  
    if [ -n "$MODE" ] ; then
      chmod "$MODE" "$TO"
    fi
    break
  done
done
return 0
}

#-------------------------------------------------------------------------------
#  QueryYN
#
#  Queries for Y-es or N-o and puts the answer in ANSWER
#  The value of ANSWER will be either "Y" or "N"
#  usage : QueryYN <msg> 
#    msg is the message that will be displayed on initially the screen
#    and repeated each time the user enters an invalid value
#-------------------------------------------------------------------------------
QueryYN ()
{
while :
do
  print -n "$1"
  read ANSWER
  case "$ANSWER" in
    $N)
      ANSWER="N"
      break
      ;;
    $Y)
      ANSWER="Y"
      break
      ;;
  esac
done
}

#-------------------------------------------------------------------------------
#  QueryYN_Yes
#
#  Queries for Y-es or N-o and puts the answer in ANSWER
#  The value of ANSWER will be either "Y" or "N"
#  The default is "Y"
#  usage : QueryYN_Yes <msg> 
#    msg is the message that will be displayed on initially the screen
#    and repeated each time the user enters an invalid value
#-------------------------------------------------------------------------------
QueryYN_Yes ()
{
while :
do
  print -n "$1"
  read ANSWER
  if [ -z "$ANSWER" ] ; then
    ANSWER="Y"
    break
  fi

  case "$ANSWER" in
    $N)
      ANSWER="N"
      break
      ;;
    $Y)
      ANSWER="Y"
      break
      ;;
  esac
done
}

#-------------------------------------------------------------------------------
#  QueryYNC_Yes
#
#  Queries for Y(es), N(o) or C(ancel) and puts the answer in ANSWER.
#  The value of ANSWER will be "Y", "N" or "C".
#  The default answer is Y(es).
#  usage : QueryYNC_Yes <msg> 
#    msg is the message that will be displayed on initially the screen
#    and repeated each time the user enters an invalid value
#-------------------------------------------------------------------------------
QueryYNC_Yes ()
{
while :
do
  print -n "$1"
  read ANSWER
  if [ -z "$ANSWER" ] ; then
    ANSWER="Y"
    break
  fi

  case "$ANSWER" in
    $N)
      ANSWER="N"
      break
      ;;
    $Y)
      ANSWER="Y"
      break
      ;;
    $C)
      ANSWER="C"
      break
      ;;
  esac
done
}

#-------------------------------------------------------------------------------
#  QueryYNC_No
#
#  Queries for Y(es), N(o) or C(ancel) and puts the answer in ANSWER.
#  The value of ANSWER will be "Y", "N" or "C".
#  The default answer is N(o).
#  usage : QueryYNC_No <msg> 
#    msg is the message that will be displayed on initially the screen
#    and repeated each time the user enters an invalid value
#-------------------------------------------------------------------------------
QueryYNC_No ()
{
while :
do
  print -n "$1"
  read ANSWER
  if [ -z "$ANSWER" ] ; then
    ANSWER="N"
    break
  fi

  case "$ANSWER" in
    $N)
      ANSWER="N"
      break
      ;;
    $Y)
      ANSWER="Y"
      break
      ;;
    $C)
      ANSWER="C"
      break
      ;;
  esac
done
}

#-------------------------------------------------------------------------------
#  QueryLicense
#-------------------------------------------------------------------------------
QueryLicense ()
{
Ini_Section_ReadMsg "$MSGFILE" LicenseBanner
while :
do
  print -n "`Ini_Section_ReadMsg "$MSGFILE" QueryLicense`"
  read ANSWER
  case "$ANSWER" in
    $NO)
      ANSWER=N
      break
      ;;
    $YES)
      ANSWER=Y
      break
      ;;
  esac
done
}

#-------------------------------------------------------------------------------
#  DetectMachineType
#-------------------------------------------------------------------------------
DetectMachineType ()
  {
  unset MACHINETYPE
  UNAME=`uname -s`

  SINIXPRESENT="`echo "$UNAME" | grep "SINIX" | wc -l | sed 's/ //g'`"
  if [ "$SINIXPRESENT" -gt 0 ] ; then
#Just because SINIX has SINIX-? characters for its UNAME e.g. SINIX-N, SINIX-Y
    UNAME="SINIX"
  fi

  case $UNAME in
    "AIX")
    VERSION=`uname -v`
    if [ "$VERSION" = "4" ] ; then
      MACHINETYPE="AIX4"
    fi
    ;;

    "HP-UX")
    VERSION=`uname -r | sed 's/..\(..\).*/\1/'`
    if [ "$VERSION" = "11" ] ; then
      MACHINETYPE="HPUX11"
    fi
    ;;
        
    "SunOS")
    VERSION=`uname -r | sed 's/\(.\).*/\1/'`
    if [ "$VERSION" = "5" ] ; then
      MACHINETYPE="Solaris"
    fi
    ;;

    "SINIX")
#Provision to detect 5.43 along with 5.42
    VERSION=`uname -r | sed 's/\(...\).*/\1/'`
    if [ "$VERSION" = "5.4" ] ; then
      MACHINETYPE="SINIX-N"
    fi
    ;;

    "dgux")
    VERSION=`uname -r` 
    processor=`uname -p`
    if [ "$processor" = "Pentium" ] ; then
         MACHINETYPE="DGINTEL"
    else
         MACHINETYPE="DGRISC"
    fi
    ;;
                   
    "OSF1")
    VERSION=`uname -r | sed 's/\(..\).*/\1/'`
    if [ "$VERSION" = "V3" ] ; then
      MACHINETYPE="OSF1_3" ;
    elif [ "$VERSION" = "V4" ] ; then
      MACHINETYPE="OSF1_4" ;
    elif [ "$VERSION" = "V5" ] ; then
      MACHINETYPE="OSF1_5" ;
    fi
    ;;
                  
    *)
    return 1
    ;;
  esac
  return 0
  }

#-------------------------------------------------------------------------------
#  Profile_ListVars <profile> <variable>
#    Reads vars from ksh profile
#-------------------------------------------------------------------------------
Profile_ListVars ()
  {
  if [ -n "$2" ] ; then
    cat "$1" 2>> "$LOGFILE"                            |\
    # split lines
    awk -F';' '{ for (i = 1; i <= NF; i++) print $i }' |\
    # filter out comment lines
    grep -v '^[ 	]*#'                           |\
    # do the grep and print after '='
    egrep "(^$2=)|([ 	]+$2=)" | awk -F= '{print$2}' 
  fi
  }

#-------------------------------------------------------------------------------
#  ksh_ListVars <homedir> <variable>
#    Reads vars from .profile in homedir
#-------------------------------------------------------------------------------
ksh_ListVars ()
  {
  Profile_ListVars "$1/.profile" "$2"
  }

#-------------------------------------------------------------------------------
#  csh_ListVars <homedir> <variable>
#    Reads vars from .cshrc and .login in homedir
#-------------------------------------------------------------------------------
csh_ListVars ()
  {
  if [ -n "$2" ] ; then
    cat "$1/.cshrc" "$1/.login" 2>> "$LOGFILE"         |\
    # split lines
    awk -F';' '{ for (i = 1; i <= NF; i++) print $i }' |\
    # filter out comment lines
    grep -v '^[ 	]*#'                           |\
    # change setenv <var> to <var>=
    sed "s/setenv[ 	][ 	]*$2[ 	]/$2=/"        |\
    # do the grep and print after '='
    egrep "(^$2[ 	]*=)|([ 	]+$2[ 	]*=)" | awk -F= '{print$2}' |\
    # eat whitespace
    sed 's/^[ 	]*//'
   fi
  }

#-------------------------------------------------------------------------------
#  Etc_ListVars <file> <variable>
#    List vars in second column from /etc/syslog, or other scripts
#    where the column delimiter is whitespace.
#    Lines starting with '#' are filtered out.
#-------------------------------------------------------------------------------
Etc_ListVars ()
  {
  if [ -n "$2" ] ; then
    cat "$1" 2>> "$LOGFILE"                      |\
    # filter out comment lines
    grep -v '^[ 	]*#'                           |\
    # do the grep and print the second column
    egrep "^$2[ 	]+" | awk '{print$2}' 
  fi
  }




#===============================================================================
#  Functions
#===============================================================================

#-------------------------------------------------------------------------------
#  ConvertHexToInteger <hex byte>
#-------------------------------------------------------------------------------
ConvertHexToInteger()
{
  case $1 in
  '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9')
  return $1
  ;;

  'a' | 'A')
  return 10
  ;;

  'b' | 'B')
  return 11
  ;;

  'c' | 'C')
  return 12
  ;;

  'd' | 'D')
  return 13
  ;;

  'e' | 'E')
  return 14
  ;;

  'f' | 'F')
  return 15
  ;;

  *)
  return -1
  ;;
  esac
}

#-------------------------------------------------------------------------------
#  ConvertnHexToInteger <hex bytes> <string length>
#-------------------------------------------------------------------------------
ConvertnHexToInteger()
{
  let total=0
  let i=1
  while [ i -le $2 ]
  do
    ConvertHexToInteger `echo $1 | cut -c$i-$i`
    let num=$?
    if [ $num -eq -1 ]; then
      return -1
    fi
    let exp=1
    let j=$2-$i-1
    while [ j -ge 0 ]
    do
      let exp=exp*16
      let j=j-1
    done
    let total=\(num*exp\)+total
    let i=i+1
  done

  HEX_RETURN=$total

  return 0
}

#-------------------------------------------------------------------------------
#  QueryContinue <errorcode>
#-------------------------------------------------------------------------------
QueryContinue()
{
QueryYN_Yes "`Ini_Section_ReadMsg "$MSGFILE" QueryContinue`"
if [ $ANSWER = N ] ; then
  Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
  DoExit "$1"
fi
}

#-------------------------------------------------------------------------------
#  QueryHitEnter
#-------------------------------------------------------------------------------
QueryHitEnter()
{
Ini_Section_ReadMsg "$INFFILE" QueryHitEnter
read scratch
}

#-------------------------------------------------------------------------------
#  DoExit <errorcode>
#-------------------------------------------------------------------------------
DoExit()
  {
  echo "error code : $1" >> "$LOGFILE"
  if [ -a "$INFFILE" ] ; then
    Ini_Section_DoCommand "$INFFILE" CleanUp
    rm -f $INFFILE 2> /dev/null
  fi               
  umask $UmaskBeforeInstallation
  exit 1
  }

#-------------------------------------------------------------------------------
#  Exit_On_Error_Untar
#-------------------------------------------------------------------------------
Exit_On_Error_Untar ()
  {
  if [ $? -ne 0 ] 
  then
    Ini_Section_ReadMsg "$LSTFILE" ExitFailure
    DoExit 300
  fi
  }
  
#-------------------------------------------------------------------------------
#  Level_Backup_Name
#    Creates a name for a backup level
#  usage: Level_Backup_Name
#-------------------------------------------------------------------------------
Level_Backup_Name ()
{
if [ -a "$1" ] ; then
  let i=1
  while :
  do
    FNAME="$1_BACK.$i"
    if [ ! -a "$FNAME" ] ; then
      break
    fi
    let i=i+1
  done
echo "$FNAME"
fi
}

#-------------------------------------------------------------------------------
#  QueryMachineType
#-------------------------------------------------------------------------------
QueryMachineType ()
  {
  Ini_Section_ReadMsg "$MSGFILE" MachineNotFound
  while [ -z "$MACHINETYPE" ]
  do      
    print -n "`Ini_Section_ReadMsg "$MSGFILE" QueryMachineType`"
    read ANSWER;
    if [ -z "$ANSWER" ] ;then
      break
    fi
    MACHINETYPE=`Ini_Section_ReadKey_ksh "$MSGFILE" MachineTypes "$ANSWER"`
  done
  }

#-------------------------------------------------------------------------------
#  CheckTcpPort
#  usage : CheckTcpPort <port> 
#-------------------------------------------------------------------------------
CheckTcpPort()
{
  kTCPPORT_MIN=1024
  kTCPPORT_MAX=65534

  sName=""
  sNum=""

  echo $1 | awk '/^[0-9+-][0-9]*$/ { print $0}' | read sNum
  if [ -z "$sNum" ]; then
    Ini_Section_ReadMsg "${INFFILE}" TcpPortNotNumeric
    return 1
  fi

  typeset -i TcpPort=$1 2> /dev/null

  if [ $TcpPort -lt $kTCPPORT_MIN ]; then
    Ini_Section_ReadMsg "${INFFILE}" TcpPortTooSmall
    return 2
  fi
  if [ $TcpPort -gt $kTCPPORT_MAX ]; then
    Ini_Section_ReadMsg "${INFFILE}" TcpPortTooLarge
    return 3
  fi

  if [ "${SKIP_PORTREGISTER}" != "$FALSE" ] ; then
    # Skip port existence.
    return 0
  fi

  ${CATSERVICES} | awk '/^[^#].*'$TcpPort'\/tcp/{print $1} /^[^#].*'$TcpPort'\/udp/{print $1}' | read sName
  if [ ! -z "$sName" ]; then
    Ini_Section_ReadMsg "${INFFILE}" QueryVarsmsg1
    return 4
  fi

  return 0
}

#-------------------------------------------------------------------------------
#  CheckServiceName
#  usage : CheckServiceName <name> 
#-------------------------------------------------------------------------------
CheckServiceName()
{

  if [ ! -z "`echo \"$1\" | grep \"[.,]\"`" ]; then
    Ini_Section_ReadMsg "${INFFILE}" QueryVarsmsg3
    return 1
  fi

  if [ \( -x "${SQLNKDIR}/admin/swcla" \) -a \( -r "${SQLNKDIR}/cfg/swandm.ini" \) ]; then
    if [ -z "`${SQLNKDIR}/admin/swcla -nologo -d ${SQLNKDIR}/cfg/swandm.ini ServiceList | grep \"^$1\"`" ]; then
      return 0
    else
      Ini_Section_ReadMsg "${INFFILE}" QueryVarsmsg2
      return 1
    fi
  else
    return 0
  fi
}

#-------------------------------------------------------------------------------
#  QuerySLHome
#  usage : QuerySLHome 
#-------------------------------------------------------------------------------
QuerySLHome ()
{
#if logged on as 'sequelnk'; use it's homedir as default installation dir
myname=`id | awk '{print $1}' | sed -n 's/uid=.*(\(.*\))/\1/p'`
if [ "$myname" = "sequelnk" ] ; then
  slhome=`$CATPASSWD | grep "^sequelnk:" | awk -F: '{print$6}'`
  if [ -n "$slhome" ] ; then
    DEF_HOME="$slhome"
  fi
fi
while :
do
  Ini_Section_DoCommand "$INFFILE" SLHOMEmsg
  read SQLNKDIR
  if [ -z "$SQLNKDIR" ] ; then
    SQLNKDIR="$DEF_HOME"
  fi

  SPACE="`echo \"${SQLNKDIR}\" | grep ' '`"
  if [ ! -z "${SPACE}" ]; then
    Ini_Section_ReadMsg "$INFFILE" SLHOMEmsg8
    continue
  fi

  # Test if we are not overwriting an existing 3.5 installation
  if [ -f "${SQLNKDIR}/servermap.tcp" ] ; then
    Ini_Section_ReadMsg "$INFFILE" SLHOMEmsg3
    continue
  fi

  # Test if we are not overwriting an existing 4.x installation
  if [ -f "${SQLNKDIR}/slsrvcfg" ] ; then
    Ini_Section_ReadMsg "$INFFILE" SLHOMEmsg3
    continue
  fi

  # Test if we are not overwriting an existing 5.0 installation
  if [ -f "${SQLNKDIR}/cfg/swandm.ini" ] ; then
    INSTALLED_DMVERSION=`Ini_Section_ReadKey "${SQLNKDIR}/cfg/swandm.ini" SwanDM DmVersion`
    INSTALLER_DMVERSION=`Ini_Section_ReadKey "${SOURCEDIR}/swandm.ini" SwanDM DmVersion`
    if [ "${INSTALLED_DMVERSION}" != "${INSTALLER_DMVERSION}" ]; then
      Ini_Section_ReadMsg "$INFFILE" SLHOMEmsg3
      continue
    fi
    DO_FULL_INSTALL=${FALSE}
  fi
  
  break
done
}

#-------------------------------------------------------------------------------
# QueryDBMSUser
# usage:
#   QueryDBMSUser <default user>
#-------------------------------------------------------------------------------
QueryDBMSUser()
{
DEF_USER="$1"
while :
do
  Ini_Section_DoCommand "$INFFILE" DBMSUSERmsg1
  read DBMS_USER
  if [ -z "$DBMS_USER" ] ; then
    DBMS_USER="$1"
  fi
  $CATPASSWD  2> /dev/null | grep "^$DBMS_USER:" 2> /dev/null 1> /dev/null
  if [ $? -ne 0 ] ; then
    Ini_Section_DoCommand "$INFFILE" DBMSUSERmsg2
    QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
    if [ "$ANSWER" = "Y" ] ; then
      continue
    fi
  fi
  break
done
}

#-------------------------------------------------------------------------------
# DetectDBMSHome
# usage:
#   DetectDBMSHome <variable name>
#-------------------------------------------------------------------------------
DetectDBMSHome()
{
Ini_Section_DoCommand "$INFFILE" DBMSHOMEmsg1
HOMEDIR=`$CATPASSWD 2>> "$LOGFILE" | grep "^$DBMS_USER:" | awk -F: '{print$6}'`
CSHELL=`$CATPASSWD 2> /dev/null | grep "^$DBMS_USER:" | awk -F: '{print$7}' | fgrep '/csh'`
Log_WriteVar "$LOGFILE" CSHELL
Log_WriteVar "$LOGFILE" HOMEDIR
if [ -n "$CSHELL" ] ; then
  list=`csh_ListVars "$HOMEDIR" "$1"`
else
  list=`ksh_ListVars "$HOMEDIR" "$1"`
fi
nfound=`echo $list | awk '{print NF}'`

if [ $nfound -eq 0 ] ; then
  DBMS_HOMEDIR=""
elif [ $nfound -eq 1 ] ; then
  DBMS_HOMEDIR=`echo $list | awk '{print $1}'`
  Ini_Section_DoCommand "$INFFILE" TestDBMSHome
  if [ $? -ne 0 ] ; then
    DBMS_HOMEDIR=""
  fi
else
  DB_DEFAULTHOME=`echo $list | awk '{print $1}'`
  while true
  do
    Ini_Section_DoCommand "$INFFILE" ListDBMSHome
    Ini_Section_DoCommand "$INFFILE" QueryDBMSHome
    read DBMS_HOMEDIR
    if [ -z "$DBMS_HOMEDIR" ]; then
      DBMS_HOMEDIR="$DB_DEFAULTHOME"
    fi

    Ini_Section_DoCommand "$INFFILE" TestDBMSHome
    if [ $? -ne 0 ] ; then
      Ini_Section_DoCommand "$INFFILE" InvalidDBMSHome
      QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" DBMSHOMEmsg2`"

      if [ $ANSWER = "Y" ] ; then
        break
      elif [ $ANSWER = "N" ] ; then
        continue
      elif [ $ANSWER = "C" ] ; then
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 208
      fi
    fi
    break
  done
fi
}

#-------------------------------------------------------------------------------
# QueryDBMSHome
#   Queries for the DBMS home directory.
#-------------------------------------------------------------------------------
QueryDBMSHome()
{
while :
do
  Ini_Section_DoCommand "$INFFILE" QueryDBMSHome
  read DBMS_HOMEDIR
  if [ -z "$DBMS_HOMEDIR" ]; then
    DBMS_HOMEDIR="$DB_DEFAULTHOME"
  fi
  Ini_Section_DoCommand "$INFFILE" TestDBMSHome
  if [ $? -ne 0 ] ; then
    Ini_Section_DoCommand "$INFFILE" InvalidDBMSHome
    QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" DBMSHOMEmsg2`"

    if [ $ANSWER = "Y" ] ; then
      break
    elif [ $ANSWER = "N" ] ; then
      continue
    elif [ $ANSWER = "C" ] ; then
      Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
      DoExit 209
    fi
  else
    break
  fi
done
}

#-------------------------------------------------------------------------------
# QueryODBCDrivers
#   Queries for the ODBC drivers directory (Socket only!).
#-------------------------------------------------------------------------------
QueryODBCDrivers()
{
while :
do
  if [ "$MACHINETYPE" = HPUX11 ]
  then
    Ini_Section_DoCommand "$INFFILE" QueryHPUXSocket
  fi
  Ini_Section_DoCommand "$INFFILE" QueryODBCDrivers
  read ODBCHOME
  if [ -z "$ODBCHOME" ]; then
    ODBCHOME="$DB_DEFAULTHOME"
  fi
  Ini_Section_DoCommand "$INFFILE" TestODBCDrivers
  if [ $? -ne 0 ] ; then
    Ini_Section_DoCommand "$INFFILE" ODBCDriversmsg1
    QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" ODBCDriversmsg2`"

    if [ $ANSWER = "Y" ] ; then
      break
    elif [ $ANSWER = "N" ] ; then
      continue
    elif [ $ANSWER = "C" ] ; then
      Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
      DoExit 210
    fi
  else
    break
  fi
done
}

#-------------------------------------------------------------------------------
# QueryDBMSVersion
#-------------------------------------------------------------------------------
QueryDBMSVersion()
  {
  Ini_Section_DoCommand "$INFFILE" QueryDBMSVer
  Ini_Section_ReadMsg "$INFFILE" ListDBMSVer
  while :
  do
    print -n "`Ini_Section_ReadMsg "$INFFILE" SelDBMSVer`"
    read ANSWER
    DBMS_VERSION=`Ini_Section_ReadKey "$INFFILE" VersionList "$ANSWER"`
    if [ -n "$DBMS_VERSION" ] ; then
      break
    fi
  done
  }
  
#-------------------------------------------------------------------------------
# QueryDBMSVars
# purpose:   
#    This function queries /detects DBMS-specific values like 
#    INFORMIXSERVER for Informix 7.x, ORACLE_SID for Oracle
#    and DB2INSTANCE for DB2UDB.
#    For the ODBC Socket, this function is used to select which databases to
#    support.
#-------------------------------------------------------------------------------
QueryDBMSVars()
  {
  case "$DBMS_ID" in
  'DB2UDB')
  QueryDB2UDBStuff
  ;;
  'Informix')
  QueryInformixStuff
  ;;
  'OpenIngres')
  QueryOpenIngresStuff
  ;;
  'Oracle')
  QueryOracleStuff
  ;;
  'Socket')
  QuerySupportedDBMS
  ;;
  'Sybase')
  QuerySybaseStuff
  ;;

  *)
  ;;
  esac

  if [ $? -eq 1 ] ; then
    Ini_Section_ReadMsg "$LSTFILE" ExitAbort
    DoExit 211
  fi
 }

#-------------------------------------------------------------------------------
# QueryDB2UDBStuff
#-------------------------------------------------------------------------------
QueryDB2UDBStuff()
  {
  Ini_Section_DoCommand "$INFFILE" DB2CSVar1

  if [ -n "$CSHELL" ] ; then
    DB2INSTANCE=""
  else
    list=`Profile_ListVars "$DBMS_HOMEDIR/sqllib/db2profile" DB2INSTANCE`
    DB2INSTANCE=`echo $list | awk '{print $1}'`
  fi

  if [ -n "$DB2INSTANCE" ] ; then
    Ini_Section_DoCommand "$INFFILE" DB2CSVar2

    QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryCorrect`"
    if [ "$ANSWER" = N ] ; then
      DEF_DB2INSTANCE=$DB2INSTANCE
      DB2INSTANCE=""
    fi
  fi

  if [ -z "$DB2INSTANCE" ] ; then
    Ini_Section_DoCommand "$INFFILE" DB2CSVar3
    read DB2INSTANCE
    if [ -z "$DB2INSTANCE" ] ; then
      DB2INSTANCE="$DEF_DB2INSTANCE"
    fi
  fi

  export DB2INSTANCE

  return 0
  }

#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
QueryInformixStuff()
  {
  #check is sqlhosts file exists and is not empty or
  #contains any valid entry (in this case something else than just comments)
  while :
  do
    echo ""
    echo "Detecting Informix client..."
    if [ -r ${DBMS_HOMEDIR}/msg/citoxmsg.pam -a -d ${DBMS_HOMEDIR}/lib/esql ]; then
      client="YES"
    else
      client="NO"
    fi
    list=`cat "$DBMS_HOMEDIR/etc/sqlhosts" 2> /dev/null |\
          grep -v '^[ \t]*#' | awk '{print $1}'`
    valid=`echo "$list"`
    if [ ! -s $DBMS_HOMEDIR/etc/sqlhosts -o -z "$valid" -o ! "$client" = "YES" ] ; then
      Ini_Section_DoCommand "$INFFILE" InvalidDBMSHome
      QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" DBMSHOMEmsg2`"

      if [ $ANSWER = "Y" ] ; then
        break
      elif [ $ANSWER = "N" ] ; then
        QueryDBMSHome
        Ini_Section_DoCommand "$INFFILE" DetectDBMSVer
      elif [ $ANSWER = "C" ] ; then
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 213
      fi
    else
      break
    fi
  done

  #get the onipcstr entry from /etc/sqlhosts
  list=`cat "$DBMS_HOMEDIR/etc/sqlhosts" 2>> "$LOGFILE" | grep -v '^[ \t]*#' |\
        grep 'onipcstr' | awk '{print $1}' | grep -v '^[ \t]*demo_on'`
  DEF_INFSERVER=`echo $list | awk '{print $1}'`

  #if this fails, get from environment
  if [ -z "$DEF_INFSERVER" ] ; then
    if [ -n "$CSHELL" ] ; then
      list=`csh_ListVars "$HOMEDIR" INFORMIXSERVER`
    else
      list=`ksh_ListVars "$HOMEDIR" INFORMIXSERVER`
    fi
    DEF_INFSERVER=`echo $list | awk '{print $1}'`
    valid=`cat "$DBMS_HOMEDIR/etc/sqlhosts" 2> /dev/null | grep -v '^[ \t]*#' |\
           awk '{print $1}' | grep "^$DEF_INFSERVER$"`
    if [ -z "$valid" ] ; then
      unset DEF_INFSERVER
    fi
  fi

  #if this also fails, take the first onsoc entry from sqlhosts 
  if [ -z "$DEF_INFSERVER" ] ; then
    list=`cat "$DBMS_HOMEDIR/etc/sqlhosts" 2> /dev/null | grep -v '^[ \t]*#' |\
          grep 'onsoc' | awk '{print $1}'`
    DEF_INFSERVER=`echo $list | awk '{print $1}'`
  fi

  #if this also fails, take the first ontli entry from sqlhosts 
  if [ -z "$DEF_INFSERVER" ] ; then
    list=`cat "$DBMS_HOMEDIR/etc/sqlhosts" 2> /dev/null | grep -v '^[ \t]*#' |\
          grep 'ontli' | awk '{print $1}'`
    DEF_INFSERVER=`echo $list | awk '{print $1}'`
  fi

  list=`cat "$DBMS_HOMEDIR/etc/sqlhosts" 2> /dev/null | grep -v '^[ \t]*#' |\
      awk '{print $1}'`
  Ini_Section_DoCommand "$INFFILE" INFVar1
  cat "$DBMS_HOMEDIR/etc/sqlhosts" 2> /dev/null | fgrep -v '#' | more |\
      awk 'BEGIN {OFS = "	"} {print $1, $2}'
  Ini_Section_DoCommand "$INFFILE" INFVar2
  while true
  do
    Ini_Section_DoCommand "$INFFILE" INFVar3
    read INFORMIXSERVER
    if [ -z "$INFORMIXSERVER" ] ; then
      INFORMIXSERVER="$DEF_INFSERVER"
    fi
  
    valid=`echo "$list" | grep "^$INFORMIXSERVER$"`
    if [ -z "$valid" ] ; then
      Ini_Section_DoCommand "$INFFILE" INFVar4
      QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" INFVar5`"
      if [ "$ANSWER" = Y ] ; then
        break
      elif [ "$ANSWER" = N ] ; then
        continue
      else
        return 1
      fi
    fi
    break 1
  done
          
  export INFORMIXSERVER

  return 0
  }

#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
QueryOpenIngresStuff()
  {
  return 0
  }

#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
QueryOracleStuff()
  {
  Ini_Section_DoCommand "$INFFILE" ORAVar1
  if [ -n "$CSHELL" ] ; then
    list=`csh_ListVars "$HOMEDIR" ORACLE_SID`
  else
    list=`ksh_ListVars "$HOMEDIR" ORACLE_SID`
  fi
  if [ -n "$list" ] ; then
    DEF_ORACLE_SID=`echo $list | awk '{print $1}'`
    ORACLE_SID="$DEF_ORACLE_SID"
    Ini_Section_DoCommand "$INFFILE" ORAVar2
  else
    unset DEF_ORACLE_SID
    unset ORACLE_SID
  fi

  while true
  do
    Ini_Section_DoCommand "$INFFILE" ORAVar3
    read ORACLE_SID
    if [ -z "$ORACLE_SID" ] ; then
      ORACLE_SID="$DEF_ORACLE_SID"
    fi

    if [ ! -f $DBMS_HOMEDIR/dbs/init$ORACLE_SID.ora ] ; then
      Ini_Section_DoCommand "$INFFILE" ORAVar4

      QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" ORAVar5`"
      if [ "$ANSWER" = Y ] ; then
        break
      elif [ "$ANSWER" = N ] ; then
        continue
      else
        return 1
      fi
    fi
    break 1
  done
  export ORACLE_SID
  return 0
  }

#-------------------------------------------------------------------------------
#  QuerySupportedDBMS
#  (Socket Only) : query which DBMS to support
#-------------------------------------------------------------------------------
QuerySupportedDBMS()
  {
  Ini_Section_DoCommand "$INFFILE" QuerySupportedDBMS
  }

#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
QuerySybaseStuff()
  {
  DEF_SYBNETWORKADDRESS="`uname -n`,4100"
  SybaseNetworkAdress="${DEF_SYBNETWORKADDRESS}"
  export SybaseNetworkAdress

  # Check if interfaces file exists.
  if [ ! -r ${DBMS_HOMEDIR}/interfaces ]; then
    Ini_Section_DoCommand "$INFFILE" SYBVar8
    read SybaseNetworkAdress
    if [ -z "${SybaseNetworkAdress}" ] ; then
      SybaseNetworkAdress="${DEF_SYBNETWORKADDRESS}"
    fi
    return 0
  fi
  
  #Show a list of all Sybase server (DSQUERY's) from the interfaces file
  Ini_Section_DoCommand "$INFFILE" SYBVar1
  cat "$DBMS_HOMEDIR"/interfaces 2>> "$LOGFILE" | grep -v '^#' | grep -v '^[ 	]' |\
        grep -v '^$' | awk '{print $1}'
  Ini_Section_DoCommand "$INFFILE" SYBVar2
  list=`cat "$DBMS_HOMEDIR"/interfaces 2> /dev/null | grep -v '^#' | grep -v '^[ 	]' |\
        grep -v '^$' | awk '{print $1}'`

  # get the default Sybase server (DSQUERY) from the environment
  if [ -n "$CSHELL" ] ; then
    deflist=`csh_ListVars "$HOMEDIR" DSQUERY`
  else
    deflist=`ksh_ListVars "$HOMEDIR" DSQUERY`
  fi
  DEF_DSQUERY=`echo $deflist | awk '{print $1}'`
  valid=`echo "$list" | grep "^$DEF_DSQUERY$"`
  if [ -z "$valid" ] ; then
    unset DEF_DSQUERY
  fi

  while true
  do
    Ini_Section_DoCommand "$INFFILE" SYBVar3
    read DSQUERY
    if [ -z "$DSQUERY" ] ; then
      DSQUERY="$DEF_DSQUERY"
    fi
  
    valid=`echo "$list" | grep "^$DSQUERY$"`
    if [ -z "$valid" ] ; then
      Ini_Section_DoCommand "$INFFILE" SYBVar4
      QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" SYBVar5`"
      if [ "$ANSWER" = Y ] ; then
        break
      elif [ "$ANSWER" = N ] ; then
        continue
      else
        return 1
      fi
    fi
    break 1
  done
  export DSQUERY

  masterline=`tail +\`cat ${DBMS_HOMEDIR}/interfaces 2> /dev/null | grep -n "^${DSQUERY}[ 	]*$" | sed -n 's-^\([0-9]*\):.*-\1-p'\` ${DBMS_HOMEDIR}/interfaces 2> /dev/null | grep '^[	]*master' | head -1`
  if [ -z "${masterline}" ]; then
    Ini_Section_DoCommand "$INFFILE" SYBVar6
    QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" SYBVar7`"
    if [ "$ANSWER" = Y ] ; then
      SybaseNetworkAdress=""
    elif [ "$ANSWER" = N ] ; then
      Ini_Section_DoCommand "$INFFILE" SYBVar8
      read SybaseNetworkAdress
      if [ -z "${SybaseNetworkAdress}" ] ; then
        SybaseNetworkAdress="${DEF_SYBNETWORKADDRESS}"
      fi
    else
      return 1
    fi
  fi

  if [ `echo ${masterline} | awk '{print $5}' | cut -c1-2` = "\x" ]; then
    masterline_port=`echo ${masterline} | awk '{print $5}' | cut -c7-10`
    masterline_machine=`echo ${masterline} | awk '{print $5}' | cut -c11-18`

    export HEX_RETURN=0

    ConvertnHexToInteger `echo ${masterline_machine} | cut -c1-2` 2
  		SybaseNetworkAdress=${HEX_RETURN}

    ConvertnHexToInteger `echo ${masterline_machine} | cut -c3-4` 2
  		SybaseNetworkAdress=${SybaseNetworkAdress}"."${HEX_RETURN}

    ConvertnHexToInteger `echo ${masterline_machine} | cut -c5-6` 2
  		SybaseNetworkAdress=${SybaseNetworkAdress}"."${HEX_RETURN}

    ConvertnHexToInteger `echo ${masterline_machine} | cut -c7-8` 2
  		SybaseNetworkAdress=${SybaseNetworkAdress}"."${HEX_RETURN}

    ConvertnHexToInteger ${masterline_port} 4
  		SybaseNetworkAdress=${SybaseNetworkAdress}","${HEX_RETURN}
  else
    SybaseHost=`echo ${masterline} | awk '{print $4}'`
    SybasePort=`echo ${masterline} | awk '{print $5}'`
    SybaseNetworkAdress=${SybaseHost}","${SybasePort}
  fi

  return 0
  }

#-------------------------------------------------------------------------------
# QueryDBMSLocaleStuff
# purpose: 
#    For using Double Byte Charcter on RDBMSs.
#    This function queries /detects DBMS-specific Japanese values like 
#    DB_LOCALE, SERVER_LOCALE and CLIENT_LOCALE for Informix 7.x,
#    and NLS_LANG for Oracle. 
#-------------------------------------------------------------------------------
QueryDBMSLocaleStuff()
  {
  if [ "$LOCALE" = Japanese ] ; then
    Ini_Section_DoCommand "$INFFILE" QueryDBMSLocaleStuff
    if [ $? -eq 1 ] ; then
      Ini_Section_ReadMsg "$LSTFILE" ExitAbort
      DoExit 212
    fi
  fi
 }

#-------------------------------------------------------------------------------
# QuerySLAgentVars
#-------------------------------------------------------------------------------
QuerySLAgentVars ()
{
  clear
  Ini_Section_ReadMsg   "$INFFILE" QueryAgentmsg1
  # Read administrator user name
  while :
  do
    Ini_Section_DoCommand "${INFFILE}" QueryAgentAdministrator
    read AGENT_SERVICE_ADMINISTRATOR
    if [ -z "${AGENT_SERVICE_ADMINISTRATOR}" ]; then
      AGENT_SERVICE_ADMINISTRATOR="${AGENT_DEFAULT_ADMINISTRATOR}"
    fi
    $CATPASSWD  2> /dev/null | grep "^${AGENT_SERVICE_ADMINISTRATOR}:" 2> /dev/null 1> /dev/null
    if [ $? -ne 0 ] ; then
      if [ \( "${AGENT_SERVICE_ADMINISTRATOR}" != "everyone" \) -a \( "${AGENT_SERVICE_ADMINISTRATOR}" != "authenticated" \) ]; then
        Ini_Section_DoCommand "${INFFILE}" CreateAgentmsg9
        QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
        if [ "${ANSWER}" = "Y" ] ; then
          continue
        fi
      fi
    fi
    break
  done
  # Read Agent service name.
  while :
  do
    Ini_Section_DoCommand "${INFFILE}" QueryAgentServiceName
    read AGENT_SERVICE_NAME
    if [ -z "${AGENT_SERVICE_NAME}" ]; then
      AGENT_SERVICE_NAME="${AGENT_DEFAULT_NAME}"
    fi
    CheckServiceName "${AGENT_SERVICE_NAME}"
    if [ $? -ne 0 ] ; then
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      if [ "${ANSWER}" = "Y" ] ; then
        continue
      else
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 1
      fi
    fi
    break
  done
  # Read Agent service port.
  while :
  do
    Ini_Section_DoCommand "${INFFILE}" QueryAgentServicePort
    read AGENT_SERVICE_PORT
    if [ -z "${AGENT_SERVICE_PORT}" ]; then
      AGENT_SERVICE_PORT="${AGENT_DEFAULT_PORT}"
    fi
    CheckTcpPort "${AGENT_SERVICE_PORT}"
    if [ $? -ne 0 ] ; then
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      if [ "${ANSWER}" = "Y" ] ; then
        continue
      else
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 1
      fi
    fi
    # Check that the Agent port doesn't conflict with the DBMS service port entered earlier.
    if [ "${DBMS_SERVICE_PORT}" = "${AGENT_SERVICE_PORT}" ]; then
      Ini_Section_ReadMsg "${INFFILE}" QueryVarsmsg1
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      if [ "${ANSWER}" = "Y" ] ; then
        continue
      else
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 1
      fi
    fi
    break
  done
}

#-------------------------------------------------------------------------------
# QuerySLDBMSVars
#-------------------------------------------------------------------------------
QuerySLDBMSVars ()
{
  clear
  Ini_Section_DoCommand "${INFFILE}" QueryDBMSServicemsg1
  # Read DBMS service name.
  while :
  do
    Ini_Section_DoCommand "${INFFILE}" QueryDBMSServiceName
    read DBMS_SERVICE_NAME
    if [ -z "${DBMS_SERVICE_NAME}" ]; then
      DBMS_SERVICE_NAME="${DBMS_SERVICE_DEFAULT_NAME}"
    fi
    CheckServiceName "${DBMS_SERVICE_NAME}"
    if [ $? -ne 0 ] ; then
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      if [ "${ANSWER}" = "Y" ] ; then
        continue
      else
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 1
      fi
    fi
    break
  done
  # Read DBMS service port.
  while :
  do
    Ini_Section_DoCommand "${INFFILE}" QueryDBMSServicePort
    read DBMS_SERVICE_PORT
    if [ -z "${DBMS_SERVICE_PORT}" ]; then
      DBMS_SERVICE_PORT="${DBMS_SERVICE_DEFAULT_PORT}"
    fi
    CheckTcpPort "${DBMS_SERVICE_PORT}"
    if [ $? -ne 0 ] ; then
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      if [ "${ANSWER}" = "Y" ] ; then
        continue
      else
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 1
      fi
    fi
    # Check that the DBMS service port doesn't conflict with the Agent port entered earlier.
    if [ "${DBMS_SERVICE_PORT}" = "${AGENT_SERVICE_PORT}" ]; then
      Ini_Section_ReadMsg "${INFFILE}" QueryVarsmsg1
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      if [ "${ANSWER}" = "Y" ] ; then
        continue
      else
        Ini_Section_ReadMsg "$LSTFILE" ExitAbort | tee -a "$LOGFILE"
        DoExit 1
      fi
    fi
    break
  done
}

#-------------------------------------------------------------------------------
# CreateAgentService
#-------------------------------------------------------------------------------
CreateAgentService ()
{
  Ini_Section_DoCommand "$INFFILE" CreateAgentmsg1

  #
  # First delete all service templates.
  #
  ${ADMINROOT}/swcla -nologo -dbg -d ${SLXDATAMODEL} std all > "${CLALOGFILE}" 2>&1
  CLARetCode=$?
  rm -f ${CLA_DM_INIT_SCRIPT}
  if [ ${CLARetCode} -ne 0 ]; then
    cat "${CLALOGFILE}" | tee -a "$LOGFILE"
    Ini_Section_ReadMsg "$INFFILE" CreateAgentmsg3 | tee -a "$LOGFILE"
    DoExit 1
  fi
  rm -f "${CLALOGFILE}"

  #
  # Copy agent service template from installer datamodel
  #
  ${ADMINROOT}/swcla -nologo -dbg ServiceTemplateCopy "${AGENT_SERVICE_TEMPLATEID}" "${SOURCEDIR}/swandm.ini" "${SLXDATAMODEL}" > "${CLALOGFILE}" 2>&1
  CLARetCode=$?
  if [ ${CLARetCode} -ne 0 ]; then
    cat "${CLALOGFILE}" | tee -a "$LOGFILE"
    Ini_Section_ReadMsg "$INFFILE" CreateAgentmsg3 | tee -a "$LOGFILE"
    DoExit 1
  fi
  rm -f "${CLALOGFILE}"

  #
  # Create agent start and stop scripts.
  #
  AGENT_STARTSCRIPT="${ADMINROOT}/startSLAgent"
  AGENT_STOPSCRIPT="${ADMINROOT}/stopSLAgent"
  AGENT_CREATESCRIPT="${ADMINROOT}/clascrpt$$"
  cat >${AGENT_CREATESCRIPT} <<STOP
set more=off
set dbg=on
set IgnoreErrors=off
set AutoSave=off

# Remove all service templates
ActivateOfflineConfig "${SLXDATAMODEL}"
ServiceTemplateDelete all
SaveConfig
CloseConfig

# Copy agent service template
ServiceTemplateCopy "${AGENT_SERVICE_TEMPLATEID}" "${SOURCEDIR}/swandm.ini" "${SLXDATAMODEL}"

# Initialize datamodel and create agent service template
ActivateOfflineConfig "${SLXDATAMODEL}"
DataModelInit "`uname -n`" "${RELROOT}/bin" "${EVENTLOGROOT}" "${LOGGINGROOT}" "" ".cat" "${SHLIBPREFIX}" "${SHLIBEXT}" "${AGENT_SERVICE_ADMINISTRATOR}"
ServiceTemplateAttributeAdd "${AGENT_SERVICE_TEMPLATEID}" ServiceEnvironmentVariable 'ISLVINI=${IPEROOT}'
ServiceCreate "${AGENT_SERVICE_NAME}" "${AGENT_SERVICE_TEMPLATEID}" "${AGENT_SERVICE_PORT}"
STOP
if [ "${SKIP_PORTREGISTER}" = "$FALSE" ] ; then
  cat >>${AGENT_CREATESCRIPT} <<STOP
ServiceRegister "${AGENT_SERVICE_NAME}"
STOP
fi
cat >>${AGENT_CREATESCRIPT} <<STOP
ProfileMonitorCreate "${AGENT_SERVICE_NAME}"
ProfileMonitorEdit "${AGENT_SERVICE_NAME}" disable
ProfileMonitorEdit "${AGENT_SERVICE_NAME}" "service" "sessions started" "on"
ProfileMonitorEdit "${AGENT_SERVICE_NAME}" "service" "active sessions" "on"
ProfileMonitorEdit "${AGENT_SERVICE_NAME}" "session" "Info" "on"
ProfileEventTraceCreate "${AGENT_SERVICE_NAME}"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" disable
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "service" "Service Started" "on"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "service" "Service Stopping" "on"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "session" "Session Started" "on"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "session" "Session Stopped" "on"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "session" "Session Authenticated" "on"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "error" "Error Internal" "on"
ProfileEventTraceEdit "${AGENT_SERVICE_NAME}" "error" "Error Occured" "on"
SaveConfig
CloseConfig

Exit
STOP
  ${ADMINROOT}/swcla -nologo -i ${AGENT_CREATESCRIPT} > "${CLALOGFILE}" 2>&1
  CLARetCode=$?
  rm -f ${AGENT_CREATESCRIPT}
  if [ ${CLARetCode} -ne 0 ]; then
    cat "${CLALOGFILE}" | tee -a "$LOGFILE"
    Ini_Section_ReadMsg "$INFFILE" CreateAgentmsg3 | tee -a "$LOGFILE"
    DoExit 1
  fi
  rm -f "${CLALOGFILE}"

  Ini_Section_ReadMsg "$INFFILE" CreateAgentmsg2 | tee -a "$LOGFILE"
  cat >${AGENT_STARTSCRIPT} <<STOP
#!/bin/ksh

ISLVINI="${IPEROOT}"
export ISLVINI

LogFile="${LOGGINGROOT}/startSLAgent\$\$.log"

echo ""
echo "Sending output to \${LogFile}"
echo "Starting SequeLink Agent \"${AGENT_SERVICE_NAME}\"..."

${ADMINROOT}/swcla -nologo -d ${SLXDATAMODEL} ServiceStart "\"${AGENT_SERVICE_NAME}\"" > \${LogFile} 2>&1

exit \$?
STOP

  cat >${AGENT_STOPSCRIPT} <<STOP
echo "Stopping SequeLink Agent ${AGENT_SERVICE_NAME}..."

${ADMINROOT}/swcla -nologo -d ${SLXDATAMODEL} ServiceStop "\"${AGENT_SERVICE_NAME}\""

exit \$?
STOP

  chmod 500 ${AGENT_STARTSCRIPT} ${AGENT_STOPSCRIPT}
}

#-------------------------------------------------------------------------------
# CreateDBMSService
#-------------------------------------------------------------------------------
CreateDBMSService()
{
  Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg1

  #
  # Copy agent service template from installer datamodel
  #
  ${ADMINROOT}/swcla -nologo -dbg ServiceTemplateCopy "${DBMS_SERVICE_TEMPLATEID}" "${SOURCEDIR}/swandm.ini" "${SLXDATAMODEL}" > "${CLALOGFILE}" 2>&1
  CLARetCode=$?
  if [ ${CLARetCode} -ne 0 ]; then
    cat "${CLALOGFILE}" | tee -a "$LOGFILE"
    Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg3
    DoExit 1
  fi
  rm -f "${CLALOGFILE}"

  # Set DBMS specific env.vars. in service template.
  CLA_DM_INIT_SCRIPT="${ADMINROOT}/clascrpt$$"
  cat /dev/null > ${CLA_DM_INIT_SCRIPT}
  cat >${CLA_DM_INIT_SCRIPT} <<STOP
set more=off
set dbg=on
set IgnoreErrors=off
set AutoSave=off

ServiceTemplateAttributeAdd "${DBMS_SERVICE_TEMPLATEID}" ServiceEnvironmentVariable 'ISLVINI=${IPEROOT}'
STOP
  if [ ! -z "${LIBPATHVAL}" ]; then
    cat >>${CLA_DM_INIT_SCRIPT} <<STOP
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable '${LIBPATHKEY}=${LIBPATHVAL}'
STOP
  fi
  case "$DBMS_ID" in
  'DB2UDB')
  cat >>${CLA_DM_INIT_SCRIPT} <<STOP
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'DB2INSTANCE=${DB2INSTANCE}'
STOP
  ;;
  'Informix')
  cat >>${CLA_DM_INIT_SCRIPT} <<STOP
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'INFORMIXDIR=${DBMS_HOMEDIR}'
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'INFORMIXSERVER=${INFORMIXSERVER}'
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'DELIMIDENT='
STOP
  ;;
  'OpenIngres')
  ;;
  'Oracle')
  cat >>${CLA_DM_INIT_SCRIPT} <<STOP
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'ORACLE_HOME=${DBMS_HOMEDIR}'
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'ORACLE_SID=${ORACLE_SID}'
STOP
  ;;
  'Socket')
  ;;
  'Sybase')
  cat >>${CLA_DM_INIT_SCRIPT} <<STOP
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} ServiceEnvironmentVariable 'ODBCINI=${BINROOT}/.odbc/odbc.ini'
ServiceTemplateAttributeAdd ${DBMS_SERVICE_TEMPLATEID} DataSourceSYBNetworkAddress '${SybaseNetworkAdress}'
STOP
  ;;

  *)
  ;;
  esac
  cat >>${CLA_DM_INIT_SCRIPT} <<STOP

SaveConfig
Exit
STOP
  ${ADMINROOT}/swcla -nologo -d ${SLXDATAMODEL} -i ${CLA_DM_INIT_SCRIPT} > "${CLALOGFILE}" 2>&1
  CLARetCode=$?
  rm -f ${CLA_DM_INIT_SCRIPT}
  if [ ${CLARetCode} -ne 0 ]; then
    cat "${CLALOGFILE}" | tee -a "$LOGFILE"
    Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg3
    DoExit 1
  fi
  rm -f "${CLALOGFILE}"

  # Create service and start scripts.
  DBMS_CREATESCRIPT="${ADMINROOT}/clascrpt$$"
  cat >${DBMS_CREATESCRIPT} <<STOP
set dbg=on
set more=off
set IgnoreErrors=off
DataModelInit "`uname -n`" "${RELROOT}/bin" "${EVENTLOGROOT}" "${LOGGINGROOT}" "" ".cat" "${SHLIBPREFIX}" "${SHLIBEXT}" ""
ServiceCreate "${DBMS_SERVICE_NAME}" "${DBMS_SERVICE_TEMPLATEID}" "${DBMS_SERVICE_PORT}"
STOP
if [ "${SKIP_PORTREGISTER}" = "$FALSE" ] ; then
  cat >>${DBMS_CREATESCRIPT} <<STOP
ServiceRegister "${DBMS_SERVICE_NAME}"
STOP
fi
cat >>${DBMS_CREATESCRIPT} <<STOP
ProfileMonitorCreate "${DBMS_SERVICE_NAME}"
ProfileMonitorEdit "${DBMS_SERVICE_NAME}" disable
ProfileMonitorEdit "${DBMS_SERVICE_NAME}" "service" "sessions started" "on"
ProfileMonitorEdit "${DBMS_SERVICE_NAME}" "service" "active sessions" "on"
ProfileMonitorEdit "${DBMS_SERVICE_NAME}" "session" "Info" "on"
ProfileEventTraceCreate "${DBMS_SERVICE_NAME}"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" disable
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "service" "Service Started" "on"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "service" "Service Stopping" "on"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "session" "Session Started" "on"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "session" "Session Stopped" "on"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "session" "Session Authenticated" "on"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "error" "Error Internal" "on"
ProfileEventTraceEdit "${DBMS_SERVICE_NAME}" "error" "Error Occured" "on"
Exit
STOP
  ${ADMINROOT}/swcla -nologo -d ${SLXDATAMODEL} -i ${DBMS_CREATESCRIPT} > "${CLALOGFILE}" 2>&1
  CLARetCode=$?
  rm -f ${DBMS_CREATESCRIPT}
  if [ ${CLARetCode} -ne 0 ]; then
    cat "${CLALOGFILE}" | tee -a "$LOGFILE"
    Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg3
    DoExit 1
  fi
  rm -f "${CLALOGFILE}"
  Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg2
}

#-------------------------------------------------------------------------------
# StartDBMSService
#-------------------------------------------------------------------------------
StartDBMSService()
{
  # Start DBMS server.
  typeset -x START_SERVICE_NAME="${DBMS_NAME}"
  QueryYN_Yes "`Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg4`"
  if [ $ANSWER = "Y" ] ; then
    while :
    do
      Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg5
      ${ADMINROOT}/swcla -nologo -l -uid "${AGENT_SERVICE_ADMINISTRATOR}" ServiceStart "\"${DBMS_SERVICE_NAME}\""
      if [ $? -eq 0 ]; then
        Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg6
        break
      else
        # DBMS server did not start.
        Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg7
        QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
        if [ "$ANSWER" = "Y" ] ; then
          continue
        fi
        DoExit 1
      fi
    done
  else
    # Don't start DBMS service.
    Ini_Section_DoCommand "$INFFILE" CreateDBMSmsg8
  fi
}

#-------------------------------------------------------------------------------
# StartAgentService
#-------------------------------------------------------------------------------
StartAgentService ()
{
  typeset -x START_SERVICE_NAME="Agent"
  QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" CreateAgentmsg4`"
  if [ $ANSWER = "Y" ] ; then
    while :
    do
    # Start Agent.
      ${AGENT_STARTSCRIPT}
      if [ $? -eq 0 ]; then
        # Agent started successfully.
        Ini_Section_DoCommand "$INFFILE" CreateAgentmsg6
        # Start DBMS server.
        StartDBMSService
        break
      else
        # Agent did not start.
        Ini_Section_ReadMsg "$INFFILE" CreateAgentmsg7 | tee -a "$LOGFILE"
        QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
        if [ "$ANSWER" = "Y" ] ; then
          continue
        fi
        DoExit 1
      fi
    done
  else
    # No agent started.
    Ini_Section_DoCommand "$INFFILE" CreateAgentmsg8
  fi
}

#-------------------------------------------------------------------------------
# CreateUninstallScript
#-------------------------------------------------------------------------------
CreateUninstallScript()
{
  UNINSTALL_SCRIPT_NAME="${RELROOT}/.sluninstall"
  UNINSTALL_CLA_SCRIPT="${RELROOT}/.clascrpt"
  cat >${UNINSTALL_SCRIPT_NAME} <<STOP
cat /dev/null > ${UNINSTALL_CLA_SCRIPT}
echo "set dbg=on" >> ${UNINSTALL_CLA_SCRIPT}
echo "set more=off" >> ${UNINSTALL_CLA_SCRIPT}
echo "set IgnoreErrors=off" >> ${UNINSTALL_CLA_SCRIPT}
echo "echo \"Stopping all SequeLink services...\"" >> ${UNINSTALL_CLA_SCRIPT}
echo "sst all" >> ${UNINSTALL_CLA_SCRIPT}
echo "echo \"  OK\"" >> ${UNINSTALL_CLA_SCRIPT}
echo "echo \"Unregistering all SequeLink services...\"" >> ${UNINSTALL_CLA_SCRIPT}
echo "su all" >> ${UNINSTALL_CLA_SCRIPT}
echo "echo \"  OK\"" >> ${UNINSTALL_CLA_SCRIPT}
echo "Exit" >> ${UNINSTALL_CLA_SCRIPT}

${ADMINROOT}/swcla -nologo -d ${SLXDATAMODEL} -i ${UNINSTALL_CLA_SCRIPT}
if [ $? -ne 0 ]; then
  echo "Stopping and unregistering SequeLink services failed."
  exit 1
fi
cd ${RELROOT}/..
rm -rf ${RELROOT}
echo "Uninstalling SequeLink server was successful."
STOP
  chmod 500 ${UNINSTALL_SCRIPT_NAME}
}

#-------------------------------------------------------------------------------
#  Toggle <var>
#    if var is $TRUE, this function echoes $FALSE;
#    otherwise it echoes $TRUE
#-------------------------------------------------------------------------------
Toggle ()
  {
  if [ "$1" = "$TRUE" ] ; then
    echo "$FALSE"
  else
    echo "$TRUE"
  fi
  }

#-------------------------------------------------------------------------------
#  Init_Lang
#  usage : Init_Lang <dir> <language>
#
#  Language specific initialisation
#-------------------------------------------------------------------------------
Init_Lang ()
{
Ini_Section_DoCommand "$LSTFILE" "$2"
if [ -z "$1" ] ; then
  MSGFILE="$msg"
  INFFILE="$inf"
elif [ "$1" = "/" ] ; then
  MSGFILE="/$msg"
  INFFILE="/$inf"
else
  MSGFILE="$1/$msg"
  INFFILE="$1/$inf"
fi
Ini_Section_DoCommand "$MSGFILE" InitToolkit
}

#===============================================================================
#  Entry point
#===============================================================================

UmaskBeforeInstallation=`umask`
umask 022

trap 'Ini_Section_ReadMsg install.lst ExitAbort | tee -a "$LOGFILE"; DoExit 1' 1
trap 'Ini_Section_ReadMsg install.lst ExitAbort | tee -a "$LOGFILE"; DoExit 2' 2
trap 'Ini_Section_ReadMsg install.lst ExitAbort | tee -a "$LOGFILE"; DoExit 3' 3
trap 'Ini_Section_ReadMsg install.lst ExitAbort | tee -a "$LOGFILE"; DoExit 15' 15

set +u

SOURCEDIR=`pwd`
UNCOMPRESS=uncompress
if [ "$SOURCEDIR" = "/" ] ; then
  LSTFILE="/install.lst"
  IPEDIR="/ipe"
else
  LSTFILE="$SOURCEDIR/install.lst"
  IPEDIR="$SOURCEDIR/ipe"
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Process command-line options
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
FORCE_LINK=""
SKIP_IPE=""
SKIP_PORTREGISTER=""
SEL_LOCALE=""
GEN_UNINSTALL=""
TARFILEZ=""
SQLNKDIR=""

if [ $# -eq 3 ] ; then
  OPTION="$1"
  TARFILEZ="$2"
  SQLNKDIR="$3"
else
  echo "$1" | grep '^-' > /dev/null
  if [ $? -eq 0 ] ; then
    OPTION="$1"
    TARFILEZ="$2"
  else
    TARFILEZ="$1"
    SQLNKDIR="$2"
  fi
fi

echo "$TARFILEZ" | grep '^-' > /dev/null
if [ $? -eq 0 ] ; then
  Ini_Section_ReadMsg "$LSTFILE" Usage
  exit 1
fi
 
if [ -n "$OPTION" ] ; then
  list=`echo "$OPTION" | awk '{for(i=2; i<=5; i++) print substr($1, i, 1)}'`
  for x in $list
  do
    case $x in
    'l')
    FORCE_LINK="X"
    ;;

    'n')
    SKIP_IPE="X"
    ;;

    'p')
    SKIP_PORTREGISTER="X"
    ;;

    's')
    SEL_LOCALE="X"
    ;;

    'u')
    GEN_UNINSTALL="X"
    ;;

    *)
    Ini_Section_ReadMsg "$LSTFILE" Usage
    exit 1
    ;;
    esac
  done
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Check/Ask locale
#   If TMPLANG is set by the master installer, pick it up and see if this
#   language is supported.
#   If command-line option select locale is given, always select
#   All other cases default to "English"
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LOCALE="English"
if [ -n "$TMPLANG" ] ; then
  exists=`Ini_Section_ReadKey_ksh "$LSTFILE" "$TMPLANG" msg`
  if [ -n "$exists" ] ; then
    LOCALE="$TMPLANG"
  fi
fi

if [ -n "$SEL_LOCALE" ] ; then
  Ini_Section_ReadMsg "$LSTFILE" QueryLanguage
  while :
  do
    print -n "`Ini_Section_ReadMsg "$LSTFILE" SelLanguage`"
    read ANSWER
    if [ -z "$ANSWER" ] ; then
      ANSWER="1"
    fi
    LOCALE=`Ini_Section_ReadKey_ksh "$LSTFILE" LanguageList "$ANSWER"`
    if [ -n "$LOCALE" ] ; then
      break
    fi
  done
fi

Init_Lang `pwd` "$LOCALE"
if [ -n "$FORCE_LINK" ] ; then FORCE_LINK="$TRUE"
else FORCE_LINK="$FALSE"; fi
if [ -n "$SKIP_IPE" ] ; then SKIP_IPE="$TRUE"
else SKIP_IPE="$FALSE"; fi
if [ -n "$GEN_UNINSTALL" ] ; then GEN_UNINSTALL="$TRUE"
else GEN_UNINSTALL="$FALSE"; fi
if [ -n "$SKIP_PORTREGISTER" ] ; then SKIP_PORTREGISTER="$TRUE"
else SKIP_PORTREGISTER="$FALSE"; fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Check platform
#
# Find MACHINETYPE value (if not already done by the master installer)
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
if [ -z "$MACHINETYPE" ] ; then
  DetectMachineType
  if [ -z "$MACHINETYPE" ] ; then
    QueryMachineType
    if [ -z "$MACHINETYPE" ] ; then
      Ini_Section_ReadMsg "$LSTFILE" ExitFailure
      DoExit 100
    fi
  fi
fi

LSTMACHINENAME=`Ini_Section_ReadKey_ksh "$LSTFILE" MachineList "$MACHINETYPE"`
if [ -z "$LSTMACHINENAME" ] ; then
  Ini_Section_ReadMsg $MSGFILE InvalidMachine
  DoExit 101
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Check user 'root'
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
myid=`id|sed -n 's/uid=\([0-9]*\).*/\1/p'`
myname=`id | awk '{print $1}' | sed -n 's/uid=.*(\(.*\))/\1/p'`
if [[ "$myid" != "0" ]] ; then
  Ini_Section_ReadMsg "$LSTFILE" NoRoot
  exit 1
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Initialising installer
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
NISCAT="`which niscat`"
if [ ! -z "${NISCAT}" ]; then
  if ${NISCAT} passwd.org_dir  2> /dev/null 1> /dev/null ; then
    CATPASSWD="${NISCAT} passwd.org_dir"
  elif ypcat passwd  2> /dev/null 1> /dev/null ; then
    CATPASSWD="ypcat passwd"
  else
    CATPASSWD="cat /etc/passwd"
  fi
  
  if ${NISCAT} services.org_dir  2> /dev/null 1> /dev/null ; then
    CATSERVICES="${NISCAT} services.org_dir"
  elif ypcat services  2> /dev/null 1> /dev/null ; then
    CATSERVICES="ypcat services"
  else
    CATSERVICES="cat /etc/services"
  fi
else
  if ypcat passwd  2> /dev/null 1> /dev/null ; then
    CATPASSWD="ypcat passwd"
  else
    CATPASSWD="cat /etc/passwd"
  fi
  
  if ypcat services  2> /dev/null 1> /dev/null ; then
   CATSERVICES="ypcat services"
  else
   CATSERVICES="cat /etc/services"
  fi
fi

CLALOGFILE="/tmp/swcla$$.log"

Log_Create "/tmp/installlog."
echo '$Revision:   1.35  $'    >> "$LOGFILE"
Log_WriteVar "$LOGFILE" SOURCEDIR
Log_WriteVar "$LOGFILE" TARFILEZ
Log_WriteVar "$LOGFILE" SQLNKDIR
Log_WriteVar "$LOGFILE" FORCE_LINK
Log_WriteVar "$LOGFILE" GEN_UNINSTALL
Log_WriteVar "$LOGFILE" SKIP_IPE
Log_WriteVar "$LOGFILE" SKIP_PORTREGISTER
Log_WriteVar "$LOGFILE" LOCALE

# Resolve the TARFILEZ parameter
if [ -z "$TARFILEZ" ] ; then
  Ini_Section_ReadMsg "$LSTFILE" ProductMsg1
  let i=0
  for x in `Ini_Section_ListKeys "$LSTFILE" "$MACHINETYPE"`
  do
    y=${x%.Z}
    if [ -a "$x" ] ; then
      let i=i+1
      eval answer"$i"=$x
      print -n "$i) "; echo "`Ini_Section_ReadKey "$LSTFILE" "$MACHINETYPE" "$x"`"
    elif [ -a "$y" ] ; then
      let i=i+1
      eval answer"$i"=$y
      print -n "$i) "; echo "`Ini_Section_ReadKey "$LSTFILE" "$MACHINETYPE" "$x"`"
    fi
  done
  Ini_Section_ReadMsg "$LSTFILE" ProductMsg2
  while :
  do
    print -n "`Ini_Section_ReadMsg "$LSTFILE" ProductMsg3`"
    read ANSWER
    if [ -z "$ANSWER" ] ; then
      TARFILEZ="$answer1"
    else
      eval TARFILEZ="\$answer$ANSWER"
    fi
    if [ -n "$TARFILEZ" ] ; then
      break
    fi
  done
fi

TARFILE=${TARFILEZ%.Z}
TARFILEZ="$TARFILE.Z"
Log_WriteVar "$LOGFILE" TARFILEZ
Log_WriteVar "$LOGFILE" TARFILE

# test existance of TAR-file
if [[ ! -a "$TARFILEZ" && ! -a "$TARFILE" ]] ; then
  Ini_Section_ReadMsg "$MSGFILE" FileNotFound | tee -a "$LOGFILE"
  print "    $TARFILEZ"                       | tee -a "$LOGFILE"
  Ini_Section_ReadMsg "$LSTFILE" ExitFailure
  DoExit 102 
fi

Ini_Section_ReadMsg "$LSTFILE" Startup

# uncompress 
if [ -f "$TARFILEZ" ] ; then
  if [ -a "$TARFILE" ] ; then
    rm -f $TARFILE
  fi
  `$UNCOMPRESS $TARFILEZ 2>> "$LOGFILE"`
  if [ $? -ne 0 ] ; then
    Ini_Section_ReadMsg "$LSTFILE" ErrorUncompress
    DoExit 103
  fi
fi

# extract inf
`$TARXF $TARFILE $inf`
if [ $? -ne 0 ] ; then
  Ini_Section_ReadMsg "$LSTFILE" ErrorUntar
  DoExit 104
fi

# See if the extracted inf is good
VALID=`Ini_Section_ReadKey "$INFFILE" Admin Identification`
if [ -z "$VALID" ] ; then
  Ini_Section_ReadMsg "$LSTFILE" InfNotfound
  DoExit 105
fi

# extract datamodel
`$TARXF $TARFILE swandm.ini`
if [ $? -ne 0 ] ; then
  Ini_Section_ReadMsg "$LSTFILE" ErrorUntar
  DoExit 104
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# DBMS initializations
#
# Important: do NOT change the sequence of these initializations
# First machine specific ("$MACHINETYPE"), then common (Init), then DBMS 
# specific (InitDBMS) and then DBMS-version specific (InitVersion).
# At last, InitInstall is done just before the actuall installation starts.
# After InitVersion, the SQLNKDIR is still NOT resolved!
# It is only after InitInstall that all installation parameters are known and set
# and will not be changed anymore!
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Ini_Section_DoCommand "$INFFILE" "$MACHINETYPE"
Ini_Section_DoCommand "$INFFILE" Init
Ini_Section_DoCommand "$INFFILE" InitDBMS
Ini_Section_DoCommand "$INFFILE" InitAgent

if [ $FORCE_LINK = $TRUE ] ; then
  DO_LINK=$TRUE
fi

Log_WriteVar "$LOGFILE" MACHINETYPE
Log_WriteVar "$LOGFILE" PATH

# Show product banner
Ini_Section_DoCommand $INFFILE ProductBanner | tee -a "$LOGFILE"
QueryContinue 200

# calc needed diskspace
TOTAL_SIZE="`Ini_Section_ReadKey "$INFFILE" Sizes TCP`"

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Get SequeLink Home directory
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
clear
QuerySLHome

if [ -z "$SQLNKDIR" ] ; then
  Ini_Section_ReadMsg "$LSTFILE" ExitFailure
  DoExit 200
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#  Get DBMS variables
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
unset DBMS_HOMEDIR
unset ODBCHOME

if [ "$DBMS_ID" = "Socket" ] ; then
  QueryODBCDrivers
else
  Ini_Section_DoCommand "$INFFILE" QueryDBMSUser
  Ini_Section_DoCommand "$INFFILE" DetectDBMSHome
  while :
  do
    if [ -z "$DBMS_HOMEDIR" ] ; then
      QueryDBMSHome
    fi

    if [ ! "$DBMS_ID" = "Sybase" -a ! "$DBMS_ID" = "DB2UDB" -a ! "$DBMS_ID" = "Informix" ] ; then
      # Detect DBMS version
      unset DBMS_VERSION
      Ini_Section_DoCommand "$INFFILE" DetectDBMSVer
      if [ -z "$DBMS_VERSION" ] ; then
        QueryDBMSVersion
      fi

      # Test if libs are available.
      # Why after DBMS version detection? Libs could be version dependent
      Ini_Section_DoCommand "$INFFILE" TestDBMSLibs
      if [ $? -ne 0 ] ; then
        Ini_Section_DoCommand "$INFFILE" InvalidDBMSLibs
        QueryYNC_No "`Ini_Section_ReadMsg "$INFFILE" DBMSHOMEmsg2`"
        if [ $ANSWER = "Y" ] ; then
          break
        elif [ $ANSWER = "N" ] ; then
          DBMS_HOMEDIR=""
          continue
        elif [ $ANSWER = "C" ] ; then
          Ini_Section_ReadMsg "$LSTFILE" ExitFailure
          DoExit 201
        fi
      fi
    fi
    break
  done
fi

# Query for DBMS-specific environment variables
QueryDBMSVars

# Detect DBMS version for DB2 on HP after DB2INSTANCE detection.
if [ "$DBMS_ID" = "DB2UDB" -a "$MACHINETYPE" = "HPUX11" ]; then
  unset DBMS_VERSION
  Ini_Section_DoCommand "$INFFILE" DetectDBMSVer
  if [ -z "$DBMS_VERSION" ] ; then
    QueryDBMSVersion
  fi
fi

# Query for DBMS-locale specific environment variables
QueryDBMSLocaleStuff

# Already do an InitVersion
Ini_Section_DoCommand "$INFFILE" InitVersion

# Query for SequeLink Agent stuff
if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  QuerySLAgentVars
fi

# Query for SequeLink DBMS service stuff
QuerySLDBMSVars

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Show all options to user with the possibility to change them
#
# The case is a little bit dirty because the meaning of the answer depends on
# which DBMS we are installing. 
#
# Answers:
#   1) Start installation
#   2) Change SequeLink directory
#   3) Change DBMS home directory or ODBC driver pack dir for the ODBC Socket
#   4) Change DBMS version for DB2UDB, Informix, OpenIngres, Oracle
#      Change the list of supported DBMS for ODBS Socket
#   5) Change DB2INSTANCE for DB2UDB, INFORMIXSERVER for Informix, 
#      ORACLE_SID for Oracle
#   6) Change the DBMS-specific Japanese environment variables for Oracle
#      and Informix
#   9) Abort installation
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

while :
do
  clear
  Ini_Section_DoCommand "$INFFILE" INSTmsg1
  
  Ini_Section_DoCommand "$INFFILE" INSTmsgSIZE
  Ini_Section_DoCommand "$INFFILE" INSTmsg2
  while :
  do
    print -n "`Ini_Section_ReadMsg "$INFFILE" INSTmsg3`"
    read ANSWER
    case $ANSWER in
      "1"|"")
      break 2
      ;;

      "2")
      QuerySLHome
      break 1
      ;;

      "3")
      if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
        QuerySLAgentVars
      fi
      break 1
      ;;

      "4")
      QuerySLDBMSVars
      break 1
      ;;

      "5")
      if [ "$DBMS_ID" = "Socket" ] ; then
        QueryODBCDrivers
        QueryDBMSVars
      elif [ "$DBMS_ID" = "Sybase" ]; then
        QueryDBMSHome
        QueryDBMSVars
      elif [ "$DBMS_ID" = "Informix" ]; then
        QueryDBMSHome
        QueryDBMSVars
        QueryDBMSLocaleStuff
      elif [ "$DBMS_ID" = "DB2UDB" ]; then
        QueryDBMSHome
        QueryDBMSVars
        if [ "$MACHINETYPE" = "HPUX11" ] ; then
          unset DBMS_VERSION
          Ini_Section_DoCommand "$INFFILE" DetectDBMSVer
          if [ -z "$DBMS_VERSION" ] ; then
            QueryDBMSVersion
          fi
        fi
      else
        QueryDBMSHome
        unset DBMS_VERSION
        Ini_Section_DoCommand "$INFFILE" DetectDBMSVer
        if [ -z "$DBMS_VERSION" ] ; then
          QueryDBMSVersion
        fi
        if [ "$DBMS_ID" = "Oracle" ] ; then
          HOMEDIR=""
          QueryDBMSVars
        fi
      fi
      break 1
      ;;

      "6")
      if [ "$DBMS_ID" = "Socket" ] ; then
        QueryDBMSVars
        break 1
      elif [ "$DBMS_ID" = "Sybase" ]; then
        QueryDBMSVars
        break 1
      elif [ "$DBMS_ID" = "Informix" ]; then
        break 1
      elif [ "$DBMS_ID" = "DB2UDB" ]; then
        QueryDBMSVars
        break 1
      else
        QueryDBMSVersion
        break 1
      fi
      ;;

      "7")
      if [ "$DBMS_ID" = "Informix" -o "$DBMS_ID" = "Oracle" ] ; then
        QueryDBMSVars
        break 1
      elif [ "$DBMS_ID" = "DB2UDB" -a "$MACHINETYPE" = "HPUX11" ]; then
        QueryDBMSVersion
        break 1
      else
        continue
      fi
      ;;

      "8")
      if [ "$LOCALE" = Japanese ] ; then
        if [[ "$DBMS_ID" = "Informix" || "$DBMS_ID" = "Oracle" ]] ; then
          QueryDBMSLocaleStuff
          break 1
        else
          continue
        fi
      else
        continue
      fi
      ;;

      "9")
      Ini_Section_ReadMsg "$LSTFILE" ExitAbort         | tee -a "$LOGFILE"
      DoExit 202
      ;;

      *)
      continue
      ;;
    esac
  done
done

Ini_Section_DoCommand "$INFFILE" InitVersion

while true
do
  #create dir if not exists
  dircreated=0
  if [ ! -d "$SQLNKDIR" ] ; then
    dircreated=1
    mkdir -p "$SQLNKDIR"                2>> "$LOGFILE"
    chmod 755 "$SQLNKDIR"               2>> "$LOGFILE"
    if [ $? -ne 0 ] ; then
      Ini_Section_DoCommand "$INFFILE" SLHOMEmsg4
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" SLHOMEmsg5`"
      if [ $ANSWER = "N" ] ; then
        Ini_Section_ReadMsg "$LSTFILE" ExitFailure
        DoExit 205
      else
        echo ""
        QuerySLHome
        continue
      fi
    fi
  fi

  # Test if we have enough space left on device
  cd "$SQLNKDIR"
  SQLNKDIR=`pwd`
  FREE_SPACE=`eval $DISKFREE`
  cd "$SOURCEDIR"

  if [ "$TOTAL_SIZE" -gt "$FREE_SPACE" ] ; then
    Ini_Section_DoCommand "$INFFILE" SLHOMEmsg6
    QueryYNC_Yes "`Ini_Section_ReadMsg "$INFFILE" SLHOMEmsg7`"
    if [ "$ANSWER" = "N" ] ; then
      break
    else
      if [ $dircreated -eq 1 ] ; then
        rmdir -p "$SQLNKDIR"                2>> "$LOGFILE"
      fi
    fi

    if [ "$ANSWER" = "Y" ] ; then
      echo ""
      QuerySLHome
      continue
    fi
    
    if [ "$ANSWER" = "C" ] ; then
      Ini_Section_ReadMsg "$LSTFILE" ExitAbort        | tee -a "$LOGFILE"
      DoExit 206
    fi
  fi
  break
done

# Check if agent object is still running.

if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  EXE_FILE="$SQLNKDIR/bin/$AGENT_SSPOBJECTTCP"
  `$PSLIST > $TEMP_FILE`
  grep "$EXE_FILE" "$TEMP_FILE" > /dev/null
  if [ $? -eq 0 ] ; then
    Ini_Section_DoCommand $INFFILE PSmsg1             | tee -a "$LOGFILE"
    rm -f "$TEMP_FILE"   2> /dev/null
    QueryContinue 203
  fi
  rm -f "$TEMP_FILE"   2> /dev/null
fi

# Check if object is still running.

EXE_FILE="$SQLNKDIR/bin/$SSPOBJECTTCP"
`$PSLIST > $TEMP_FILE`
grep "$EXE_FILE" "$TEMP_FILE" > /dev/null
if [ $? -eq 0 ] ; then
  Ini_Section_DoCommand $INFFILE PSmsg1               | tee -a "$LOGFILE"
  rm -f "$TEMP_FILE"   2> /dev/null
  QueryContinue 203
fi
rm -f "$TEMP_FILE"   2> /dev/null

RELROOT="$SQLNKDIR"
ADMINROOT="$RELROOT/admin"
BINROOT="$RELROOT/bin"
CFGROOT="$RELROOT/cfg"
IPEROOT="$RELROOT/ipe"
LOGGINGROOT="$SQLNKDIR/logging"
EVENTLOGROOT="$SQLNKDIR/tracing"
SLXDATAMODEL="$CFGROOT/swandm.ini"
                      
# Add DBMS specific entries for LIBPATHVAL (Oracle is handled later).
if [ "$DBMS_ID" = "Sybase" ] ; then
  LIBPATHVAL="${BINROOT}/.odbc/lib:$LIBPATHVAL"
elif [ "$DBMS_ID" = "DB2UDB" ] ; then
  LIBPATHVAL="${DBMS_HOMEDIR}/sqllib/lib:${BINROOT}:$LIBPATHVAL"
elif [ "$DBMS_ID" = "Informix" ] ; then
  LIBPATHVAL="${DBMS_HOMEDIR}/lib:${DBMS_HOMEDIR}/lib/esql:$LIBPATHVAL"
fi

Log_WriteVar "$LOGFILE" SQLNKDIR
Log_WriteVar "$LOGFILE" FREE_SPACE
Log_WriteVar "$LOGFILE" TOTAL_SIZE
Log_WriteVar "$LOGFILE" RELROOT
if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  Log_WriteVar "$LOGFILE" AGENT_SERVICE_ADMINISTRATOR
  Log_WriteVar "$LOGFILE" AGENT_SERVICE_NAME
  Log_WriteVar "$LOGFILE" AGENT_SERVICE_PORT
fi
Log_WriteVar "$LOGFILE" DBMS_SERVICE_NAME
Log_WriteVar "$LOGFILE" DBMS_SERVICE_PORT
if [ "$DBMS_ID" = "Socket" ] ; then
  Log_WriteVar "$LOGFILE" ODBCHOME
else
  Log_WriteVar "$LOGFILE" DBMS_HOMEDIR
fi
Log_WriteVar "$LOGFILE" DBMS_VERSION
if [ "$DBMS_ID" = "Informix" ] ; then
  Log_WriteVar "$LOGFILE" INFORMIXSERVER
  Log_WriteVar "$LOGFILE" DB_LOCALE
  Log_WriteVar "$LOGFILE" SERVER_LOCALE
  Log_WriteVar "$LOGFILE" CLIENT_LOCALE
fi
if [ "$DBMS_ID" = "Oracle" ] ; then
  Log_WriteVar "$LOGFILE" ORACLE_SID
  Log_WriteVar "$LOGFILE" NLS_LANG
  Log_WriteVar "$LOGFILE" ORA_NLS
fi
if [ "$DBMS_ID" = "DB2UDB" ] ; then
  Log_WriteVar "$LOGFILE" DB2INSTANCE
fi
if [ "$DBMS_ID" = "Sybase" ] ; then
  Log_WriteVar "$LOGFILE" DSQUERY
  Log_WriteVar "$LOGFILE" SybaseNetworkAdress
fi

# now that all installation options are known, we do the last initialization:
Ini_Section_DoCommand "$INFFILE" InitInstall

# In InitInstall section of all database SWHOME is been set now.
# Always keep these lines below InitInstall section (i.e. from all *.inf files)
# This variable will be used during on-site linking, of SL executables for
# mostly sybase and ingres.
SWSHLIBDIR="-L$BINROOT"
export SWSHLIBDIR
                                            
                                            
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#  Start installation
#
#  Finally, here the real installation starts: 
#  extracting files, generating scripts and templates, linking executables
#  and configurating a server.
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#  Create root directories
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

Dir_Create "$RELROOT"             755
Dir_Create "$BINROOT"             500
Dir_Create "$CFGROOT"             700
Dir_Create "$IPEROOT"             700
#Dir_Create "$ADMINROOT"           500
Dir_Create "$LOGGINGROOT"         777
Dir_Create "$EVENTLOGROOT"        755

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#  IPE registration
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

if [ "${SKIP_IPE}" = "$FALSE" ] ; then

  # Check if another version of the DBMS server wasn't already installed.
  if [ ! -f "${IPEROOT}/prodlist.ini" ]; then
    cat /dev/null > ${IPEROOT}/prodlist.ini | tee -a "$LOGFILE"
    chmod 600 ${IPEROOT}/prodlist.ini       | tee -a "$LOGFILE"
  fi

  clear
  Ini_Section_UntarFiles "$INFFILE" Ipe "$TARFILE" "$IPEROOT" 
  Exit_On_Error_Untar

  chmod 500 ${IPEROOT}/iperegister | tee -a "$LOGFILE"
  chmod 500 ${IPEROOT}/ipemerge    | tee -a "$LOGFILE"
  chmod 400 ${IPEROOT}/ipemsg.msg  | tee -a "$LOGFILE"

  # Create local prodlist.ini file.
  cat >prodlist.ini <<STOP
[${IPEDBMSTAG}_${IPEVERSIONTAG}_${IPEOSTAG}]
${IPEDBMSTAG}_${IPEVERSIONTAG}_${IPEOSTAG}=
${IPEDBMSTAG}0_${IPEVERSIONTAG}_${IPEOSTAG}=
${IPEDBMSTAG}ODBC_${IPEVERSIONTAG}_${IPEOSTAG}=
${IPEDBMSTAG}OLEDB_${IPEVERSIONTAG}_${IPEOSTAG}=
${IPEDBMSTAG}JDBC_${IPEVERSIONTAG}_${IPEOSTAG}=
${IPEDBMSTAG}XA_${IPEVERSIONTAG}_${IPEOSTAG}=
${IPEDBMSTAG}RA_${IPEVERSIONTAG}_${IPEOSTAG}=

STOP

  if [ ! -x "${IPEROOT}/ipeexeu.sh" ]; then
    cat /dev/null > ${IPEROOT}/ipeexeu.sh | tee -a "$LOGFILE"
    cat >>${IPEROOT}/ipeexeu.sh <<STOP
#!/bin/sh

ISLVINI=${IPEROOT}
export ISLVINI

cd ${IPEROOT}
./iperegister

exit \$?
STOP
    chmod 500 ${IPEROOT}/ipeexeu.sh | tee -a "$LOGFILE"
  fi

  # Merge local prodlist.ini with installation prodlist.ini.
  ${IPEROOT}/ipemerge prodlist.ini "${IPEROOT}/prodlist.ini" >> "$LOGFILE"
  rm -f prodlist.ini                                         >> "$LOGFILE"

  # Do IPE registration
  while :
  do
    # Print header.
    clear
    Ini_Section_ReadMsg "$INFFILE" IPEHeader
    # Do the IPE thing.
    ${IPEROOT}/ipeexeu.sh
    if [ $? != 0 ]; then
      Ini_Section_DoCommand "$INFFILE" IPEAgain
      QueryYN_Yes "`Ini_Section_ReadMsg "$INFFILE" QueryTryAgain`"
      case "$ANSWER" in
        $N)
        Ini_Section_ReadMsg "$LSTFILE" ExitFailure | tee -a "$LOGFILE"
        DoExit 301
        ;;
        $Y)
        continue
        ;;
      esac
    else
      break
    fi
  done
  chmod a+r "${IPEROOT}/.islvrc" | tee -a "$LOGFILE"

  QueryHitEnter

fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#  Untar files
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
clear
Ini_Section_ReadMsg "$INFFILE" INSTmsg4

if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  Ini_Section_UntarFiles "$INFFILE" SLHome              "$TARFILE" "$SQLNKDIR" 
  Exit_On_Error_Untar
  Ini_Section_UntarFiles "$INFFILE" AdminLevel          "$TARFILE" "$RELROOT" 
  Exit_On_Error_Untar
  chmod -R 700 ${ADMINROOT}
fi

# DBMS version specific files
Ini_Section_UntarFiles "$INFFILE" Home"$DBMS_VERSION" "$TARFILE" "$SQLNKDIR"
Exit_On_Error_Untar
Ini_Section_UntarFiles "$INFFILE" Rel"$DBMS_VERSION"  "$TARFILE" "$RELROOT"
Exit_On_Error_Untar
if [ "$DBMS_ID" = "Oracle" ]; then
  if [ "$DBMS_VERSION" = "73" ]; then
    Ini_Section_DoCommand "$INFFILE" ExtractSharedLibs
    LIBPATHVAL="${RELROOT}/bin:$LIBPATHVAL"
  else
    LIBPATHVAL="${DBMS_HOMEDIR}/lib:$LIBPATHVAL"
  fi
fi
if [ "$DBMS_ID" = "Sybase" ] ; then
  Ini_Section_DoCommand "$INFFILE" InstallSybaseODBC
fi
if [ "$DBMS_ID" = "DB2UDB" ]; then
  Ini_Section_DoCommand "$INFFILE" ExtractSharedLibs
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#  Installation phase for SequeLink services
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

# extract libraries
if [ "$DO_LINK" = "$TRUE" ] ; then
  # Extract common, DBMS and DBMS_VERSION specific libraries
  Ini_Section_UntarFiles "  $INFFILE" Lib                "$TARFILE" "$RELROOT"
  Exit_On_Error_Untar
  Ini_Section_UntarFiles   "$INFFILE" Lib"$DBMS_VERSION" "$TARFILE" "$RELROOT"
  Exit_On_Error_Untar
  # Initialization for on-site linking
  Ini_Section_DoCommand "$INFFILE" InitBuild
fi

# extract Agent exe and msg files
if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  Ini_Section_UntarFiles "$INFFILE" Agent "$TARFILE" "$RELROOT"
  Exit_On_Error_Untar
  
  # copy mgss libraries from cla to bin
  cp ${ADMINROOT}/libcmgss*.* ${BINROOT}
  
  # extract data model
  Ini_Section_UntarFiles "$INFFILE" DataModel "$TARFILE" "$RELROOT"
  Exit_On_Error_Untar
  chmod 600 ${SLXDATAMODEL}
  # add the datamodel to the cla config file
  chmod u+w ${ADMINROOT}/swcla.ini
  cat >>${ADMINROOT}/swcla.ini <<STOP

[SWANDM]
SWANDM=${SLXDATAMODEL}
STOP
  chmod u-w ${ADMINROOT}/swcla.ini
fi

# extract/link TCP service exe's and msg files
if [ $DO_LINK = $TRUE ] ; then
  Ini_Section_ReadMsg "$INFFILE" SBLDmsg1
  cd "$RELROOT"/bin
  ksh ./swld            1> "$TEMP_FILE".1      2> "$TEMP_FILE".2
  ERROR=$?
  cat "$TEMP_FILE".1    2> /dev/null           | tee -a "$LOGFILE"
  cat "$TEMP_FILE".2    2> /dev/null           | tee -a "$LOGFILE"
  rm -f "$TEMP_FILE".1  2> /dev/null
  rm -f "$TEMP_FILE".2  2> /dev/null
  cd "$SOURCEDIR"
  if [ $ERROR -ne 0 ] ; then
    Ini_Section_ReadMsg "$INFFILE" SBLDmsg2    | tee -a "$LOGFILE"
    Ini_Section_ReadMsg "$LSTFILE" ExitFailure | tee -a "$LOGFILE"
    DoExit 207
  fi
  QueryHitEnter
else
  Ini_Section_UntarFiles "$INFFILE" TCP"$DBMS_VERSION" "$TARFILE" "$RELROOT"
  Exit_On_Error_Untar
fi
Ini_Section_UntarFiles "$INFFILE" MSG "$TARFILE" "$RELROOT"
Exit_On_Error_Untar

chmod -R 500 ${BINROOT}/*

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create Agent service
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
clear
if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  typeset -x AGENT_STARTSCRIPT AGENT_STOPSCRIPT AGENT_SERVICE_ADMINISTRATOR
  CreateAgentService
fi

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Create DBMS service
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CreateDBMSService

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Start services
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
if [ ${DO_FULL_INSTALL} = ${TRUE} ] ; then
  StartAgentService
fi

# create optional uninstall script
if [ ${GEN_UNINSTALL} = ${TRUE} ] ; then
  CreateUninstallScript
fi

# cleanup
Ini_Section_DoCommand "$INFFILE" CleanUp
rm -f $INFFILE     2> /dev/null

# say Bye bye
Ini_Section_ReadMsg "$LSTFILE" ExitSuccess | tee -a "$LOGFILE"

exit 0

